/*
 *  Copyright (C) 2006  UCOT
 *
 *  Distributed under the terms of the MIT License.
 *  See "LICENCE" or http://www.opensource.org/licenses/mit-license.php for details.
 */
package ucot;

import java.text.MessageFormat;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

/**
 * <p>
 * This class contains interface to the localised message
 * strings.
 * </p>
 * 
 * <p>
 * Strings are readed using ResourceBundle that uses
 * <tt>ucot/message.properties</tt> file (or its localized
 * version).
 * </p>
 * 
 * @author pajumasu
 */
public class Messages {

    private static final String BUNDLE_NAME = "ucot.messages";

    private static final ResourceBundle RESOURCE_BUNDLE = ResourceBundle
            .getBundle(BUNDLE_NAME);

    /**
     * <p>
     * Returns localized string for the given key. If no
     * string is found then !key! is returned (where key is
     * the key-string that was given as parameter)
     * </p>
     * 
     * <p>
     * <tt>getString("FILE_NOT_FOUND");</tt> would return
     * some string definded for example in properties file
     * or string <tt>"!FILE_NOT_FOUND!"</tt> if no
     * matching key is located.
     * </p>
     * 
     * @param key
     *            Key for the string required.
     * @return localized string for the given key.
     */
    public static String getString(String key) {
        try {
            return RESOURCE_BUNDLE.getString(key);
        } catch (MissingResourceException e) {
            return '!' + key + '!';
        }
    }

    /**
     * <p>
     * Method formats messages with parameters to user
     * readable format. MessageString is normal string that
     * marks the places for the dynamic information with
     * {}-marks and number in between them.
     * </p>
     * 
     * <p>
     * For example:
     * <tt>formatMessage("File {0} not found.", new File("file.txt"));</tt>
     * would return string:
     * <tt>"File file.txt not found."</tt>.
     * </p>
     * 
     * <p>
     * To use with getString method you can do like this:
     * <tt>System.out.println(formatMessage(getString("FILE_NOT_FOUND"),
     * new File("file.txt")));</tt>.
     * If there is property file that contains line:
     * <tt>FILE_NOT_FOUND=File {0} not found.</tt> then it
     * would return the same string: <tt>"File file.txt
     * not found."</tt>.
     * </p>
     * 
     * @param messageString
     *            Message format string.
     * @param args
     *            Arguments for the string.
     * @return Returns string that is builded using the
     *         arguments.
     */
    public static String formatMessage(
            String messageString, Object... args) {
        MessageFormat mf = new MessageFormat(messageString);
        return mf.format(args);
    }

    /**
     * <p>
     * Gets string localized string and format the arguments
     * using it.
     * </p>
     * 
     * @see #formatMessage(String, Object[])
     * @see #getString(String)
     * @return The formated and localized string.
     */
    public static String getFormatedMessage(String key,
            Object... args) {
        MessageFormat mf = new MessageFormat(getString(key));
        return mf.format(args);
    }

    /*
     * public static void main(String[] args){
     * System.out.println(formatMessage("Terve {0} {1}
     * {2}!","panu",new Integer(1),"plj"));
     * System.out.println(formatMessage(getString("FILE_NOT_FOUND"),new
     * File("."))); }
     */

}
