/*
 *  Copyright (C) 2006  UCOT
 *
 *  Distributed under the terms of the MIT License.
 *  See "LICENCE" or http://www.opensource.org/licenses/mit-license.php for details.
 */
package ucot.core;

import java.io.IOException;
import java.net.URL;
import java.util.Vector;

import ucot.ModulePropertyInterface;
import ucot.heuristic.HeuristicInterface;
import ucot.input.InputCollection;
import ucot.input.UseCase;
import ucot.input.UseCaseCollection;
import ucot.model.AnalyzeModel;
import ucot.output.OutputInterface;
import ucot.parser.ParserInterface;

/**
 * <p>
 * This interface controls the basics of the UCOT-program
 * (starting, parsing, shutting down etc). UCOT core
 * implements this interface and it is designed in a way
 * that the user interface using the core can control the
 * execution of the program effectively.
 * </p>
 * 
 * @author vevijopi, tujupien
 */
public interface ControlInterface extends
        ModulePropertyInterface {
    public static final String PROPERTY_CURRENT_PARSER = "CURRENT_PARSER";

    public static final String PROPERTY_CURRENT_HEURISTIC = "CURRENT_HEURISTIC";

    /**
     * <p>
     * Shuts down the program. The shutdown routine triggers
     * all possible autosave actions and after that the core
     * gets rid of all its modules and prepares itself for
     * getting automatically junkbusted.
     * </p>
     */
    public void shutdown();

    /**
     * <p>
     * Returns the UCOT root directory. That is either the
     * system directory where the class files are stored
     * under a hierarchical directory structure based on the
     * package definitions or the system directory where the
     * UCOT JAR distribution package is stored. The result
     * depends solely on the fact which distribution is
     * currently used.
     * </p>
     * 
     * @return The path to the UCOT root directory as a
     *         String.
     */
    public String getRootDir();

    /**
     * <p>
     * Clears the whole current analyze model. Basically
     * this is similiar to creating a whole new empty
     * analyze model. Notice that the old analyze model is
     * not saved anywhere and the restoration of the old
     * model is impossible without manually backing up the
     * old analyze model before using this function.
     * </p>
     */
    public void clearAnalyzeModel();

    /**
     * <p>
     * Loads use cases from the given file.
     * </p>
     * 
     * @param url
     *            URL to the use case file.
     */
    public void loadUseCases(URL url);

    /**
     * <p>
     * Reloads use cases from file. First core should remove
     * all use cases that are loaded from the given url, and
     * then it should (re)read the use cases from the given
     * file.
     * </p>
     * 
     * @param url
     *            URL of the file to be reloaded.
     */
    public void reloadUseCases(URL url);

    /**
     * <p>
     * Loads serialized analyze model from the given file.
     * </p>
     * 
     * @param url
     *            URL to the file containing a serialized
     *            analyze model.
     * @throws IOException
     *             If something goes wrong while loading the
     *             analyze model from given URL.
     */
    public void loadAnalyzeModel(URL url)
            throws IOException;

    /**
     * <p>
     * Saves the current analyze model to a java
     * serialization file which location is pointed by the
     * given URL.
     * </p>
     * 
     * @param url
     *            URL to the file where the current analyze
     *            model should be serialized and saved.
     */
    public void saveAnalyzeModel(URL url);

    /**
     * <p>
     * Returns all currently loaded use cases in a
     * UseCaseCollection.
     * </p>
     * 
     * @return UseCaseColletion containing all currently
     *         loaded use cases.
     */
    public UseCaseCollection getUseCaseCollection();

    /**
     * <p>
     * Requests core to parse given use cases, perform
     * heuristic on them and add then them to current
     * analyze model.
     * </p>
     * 
     * <p>
     * Default parser and heuristic are used for this
     * operation.
     * </p>
     * 
     * @param useCases
     *            Use cases to work magic on.
     */
    public void addToAnalyzeModel(Vector<UseCase> useCases);

    /**
     * <p>
     * Requests core to parse use cases, perform heuristic
     * on them and add them to current analyze model.
     * </p>
     * 
     * <p>
     * If either the given parser or heuristic is unknown to
     * the UCOT core, then nothing is done.
     * </p>
     * 
     * @param useCases
     *            Use cases to work magic on.
     * @param parser
     *            Parser to use.
     * @param heuristic
     *            Heuristic to use.
     */
    public void addToAnalyzeModel(Vector<UseCase> useCases,
            ParserInterface parser,
            HeuristicInterface heuristic);

    /**
     * <p>
     * Method for getting a pointer to the current analyze
     * model being handled in the core.
     * </p>
     * 
     * <p>
     * Notice that all editing to the analyze model should
     * be done through the AnalyzeModelEditor which can be
     * easily acquired with the <code>getEditor()</code>
     * method from the <code>AnalyzeModel</code> itself.
     * </p>
     * 
     * @return Current analyze model.
     */
    public AnalyzeModel getAnalyzeModel();

    /**
     * <p>
     * Returns a vector containing all the parsers currently
     * available to the UCOT core.
     * </p>
     * 
     * @return Vector containing all known parser adapters
     *         as <code>ParserInterfaces</code>.
     */
    public Vector<ParserInterface> getParsers();

    /**
     * <p>
     * Returns a vector containing all the heuristics
     * currently available to the UCOT core.
     * </p>
     * 
     * @return Vector containing all known heuristic
     *         adapters as <code>HeuristicInterfaces</code>.
     */
    public Vector<HeuristicInterface> getHeuristics();

    /**
     * <p>
     * Returns a vector containing all the output adapters
     * currently available to the UCOT core.
     * </p>
     * 
     * @return Vector containing all known output adapters
     *         as <code>OutputInterfaces</code>.
     */
    public Vector<OutputInterface> getOutputs();

    /**
     * <p>
     * Returns all the input adapters currently available to
     * UCOT core.
     * </p>
     * 
     * @return InputCollection which contains all known
     *         input adapters as <code>InputInterface</code>.
     */
    public InputCollection getInputs();

    /**
     * <p>
     * Sets the default parser to be used in parsing
     * progresses.
     * </p>
     * 
     * @param parser
     *            Parser to be used by default.
     */
    public void setCurrentParser(ParserInterface parser);

    /**
     * <p>
     * Sets the default heuristic to used in analyzation
     * progresses.
     * </p>
     * 
     * @param heuristic
     *            Default heuristic to be used by default.
     */
    public void setCurrentHeuristic(
            HeuristicInterface heuristic);

    /**
     * <p>
     * Returns the current default parser.
     * </p>
     * 
     * @return Default parser as a
     *         <code>ParserInterface</code>.
     */
    public ParserInterface getCurrentParser();

    /**
     * <p>
     * Returns the current heuristic.
     * </p>
     * 
     * @return Current heuristic as a
     *         <code>HeuristicInterface</code>.
     */
    public HeuristicInterface getCurrentHeuristic();

    /**
     * <p>
     * Exports given analyze model to given URL using given
     * output adapter.
     * </p>
     * 
     * @param url
     *            Destination URL.
     * @param output
     *            <code>OutputInterface</code> of the
     *            output adapter to be used.
     * @param analyzeModel
     *            Analyze model to export.
     * @throws If
     *             something goes wrong with the output
     *             operation, then an exception is thrown.
     */
    public void output(URL url, OutputInterface output,
            AnalyzeModel analyzeModel) throws Exception;

}
