/*
 *  Copyright (C) 2006  UCOT
 *
 *  Distributed under the terms of the MIT License.
 *  See "LICENCE" or http://www.opensource.org/licenses/mit-license.php for details.
 */
package ucot.heuristic;

import java.util.List;
import java.util.Properties;
import java.util.logging.Logger;

import ucot.Messages;
import ucot.ModuleProperties;
import ucot.exceptions.BadPropertyValueException;
import ucot.model.AnalyzeModel;
import ucot.model.ModelEditor;
import ucot.parser.Link;
import ucot.parser.ParsedData;
import ucot.parser.ParserInterface;
import ucot.parser.Sentence;
import ucot.parser.Word;

/**
 * Abbott's Heuristic is a way to etsimate which parts of
 * the speech are relevant and in which way when doing
 * object analyze.
 * 
 * @author UCOT
 */
public class AbbottsHeuristic extends ModuleProperties
        implements HeuristicInterface {

    public static final String name = "Abbott's heuristic";

    private Logger logger = Logger.getLogger("ucot.parser");

    /*
     * (non-Javadoc)
     * 
     * @see ucot.heuristic.HeuristicInterface#doHeuristic(ucot.parser.ParsedData,
     *      ucot.model.AnalyzeModel)
     */
    public void doHeuristic(ParsedData data,
            AnalyzeModel analyzeModel) throws Exception {
        if (analyzeModel == null || data == null) return;

        // Signal that thing are starting to happend
        // so model observers can case updating things.
        analyzeModel.getEditor().updationStartedSignal();

        // Make data remember how it has been parsed.
        data.getUseCase().setHeuristic(this);

        ModelEditor editor = analyzeModel.getEditor();

        // Loop through all of the sences.
        for (Sentence s : data.getSentences()) {

            logger
                    .fine(Messages
                            .getFormatedMessage(
                                    "Heuristic.PARSING_SENTENCE_LOG_MESSAGE",
                                    s));

            // Loop through all the words.
            for (Word word : s.getWords()) {

                // Basic information from the word.
                String basicForm = word.getBasicForm();
                String wordClass = word.getWordClass();
                if (basicForm.length() == 0) continue;

                if (wordClass.equals(ParserInterface.NOUN)) {
                    // Word is noun. So its is probably an
                    // entity.
                    if (!editor.containsEntity(basicForm)) {
                        editor.addEntity(basicForm);
                    }
                } else if (wordClass
                        .equals(ParserInterface.VERB)) {
                    // Word is verb. So it is probably a
                    // operation
                    // between entities.

                    // Check if there exist information
                    // about the structure of the sentence.
                    Word subject = word
                            .getLinked(ParserInterface.SUBJECT);

                    if (null != subject) {
                        // We know the subject
                        // so we can add "method" to the
                        // entity.
                        String entityName = subject
                                .getBasicForm();
                        if (!editor
                                .containsEntity(entityName)) {
                            editor.addEntity(entityName);
                        }

                        String methodName = word
                                .getBasicForm();
                        if (!editor.containsMethod(
                                entityName, methodName)) {
                            editor.addMethod(entityName,
                                    methodName);
                        }

                        List<Link> links = word.getLinks();
                        for (Link link : links) {
                            // There is structural
                            // information
                            // so we can create entity to
                            // entity
                            // relation.
                            Word to = link.getTo();
                            if (!subject.equals(to)) {
                                if (!editor
                                        .containsEntity(to
                                                .getBasicForm())) {
                                    editor
                                            .addEntity(to
                                                    .getBasicForm());
                                }
                                editor
                                        .addEntityInfluenceByMethod(
                                                subject
                                                        .getBasicForm(),
                                                word
                                                        .getBasicForm(),
                                                to
                                                        .getBasicForm());
                            }
                        }
                    }

                }
            }
        }
        analyzeModel.getEditor().readySignal();
    }

    /*
     * (non-Javadoc)
     * 
     * @see ucot.heuristic.HeuristicInterface#getName()
     */
    public String getName() {
        return name;
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return this.getName();
    }

    /*
     * (non-Javadoc)
     * 
     * @see ucot.ModulePropertyInterface#loadDefaultProperties()
     */
    public Properties loadDefaultProperties() {
        Properties result = super.loadDefaultProperties();
        // TODO Set default properties here, for example:
        // result.setProperty("SETTING_1", "Woot");
        // result.setProperty("SETTING_2", "WTF?");
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see ucot.ModulePropertyInterface#applyProperties()
     */
    public void applyProperties()
            throws BadPropertyValueException {
        // TODO Auto-generated method stub

    }

}
