/*
 *  Copyright (C) 2006  UCOT
 *
 *  Distributed under the terms of the MIT License.
 *  See "LICENCE" or http://www.opensource.org/licenses/mit-license.php for details.
 */
package ucot.input;

import java.net.URL;
import java.util.Iterator;
import java.util.Vector;

import ucot.heuristic.HeuristicInterface;
import ucot.model.AnalyzeModel;
import ucot.parser.ParserInterface;

/**
 * <p>
 * UseCase class. Contains UseCaseSteps related to this
 * usecase.
 * </p>
 * 
 * @author vevijopi
 * 
 */
public class UseCase implements Iterable<UseCaseStep> {

    /**
     * <p>
     * Vector that contains all usecase steps
     * </p>
     */
    private Vector<UseCaseStep> useCaseSteps;

    private URL url;

    private String name = "";

    /**
     * <p>
     * Is usecase added to current analyze model.
     * </p>
     */
    private boolean isUseCaseAnalyzed;

    /**
     * <p>
     * Is this a sub-usecase, mainly used when connecting
     * sub-usecases to usecase steps
     * </p>
     */
    private boolean subUseCase = false;

    /**
     * <p>
     * When heuristic was ran on this usecase, this model
     * was created. It was later added to the main usecase,
     * but a copy was left. This copy is used for
     * highlighting.
     * </p>
     */
    private AnalyzeModel model = null;

    /**
     * <p>
     * Which heuristicInterface was used on this usecase
     * </p>
     */
    private HeuristicInterface heuristic = null;

    /**
     * <p>
     * Which parserInterface was used on this usecase
     * </p>
     */
    private ParserInterface parser = null;

    /**
     * <p>
     * UseCase's id loaded from processml files or from
     * simple usecase format Only used for connecting
     * usecase steps to sub-usecases
     * </p>
     */
    private String useCaseId = null;

    public UseCase() {
        useCaseSteps = new Vector<UseCaseStep>();
        url = null;
        useCaseId = new String();
        this.isUseCaseAnalyzed = false;
    }

    /**
     * <p>
     * Set the parser that was used to parse this usecase
     * </p>
     * 
     * @param parser
     *            parser that was used
     */
    public void setParser(ParserInterface parser) {
        this.parser = parser;
    }

    /**
     * <p>
     * Set the heuristic that was used to this usecase
     * </p>
     * 
     * @param heuristic
     *            heuristic
     */
    public void setHeuristic(HeuristicInterface heuristic) {
        this.heuristic = heuristic;
    }

    /**
     * <p>
     * Get the parser that was used on this usecase
     * </p>
     * 
     * @return parser that was used
     */
    public ParserInterface getParser() {
        return parser;
    }

    /**
     * <p>
     * Get the heuristic that was used on this usecase
     * </p>
     * 
     * @return heuristic that was used
     */
    public HeuristicInterface getHeuristic() {
        return heuristic;
    }

    /**
     * <p>
     * Returns name of the parser that was used in creating
     * this usecase
     * </p>
     * 
     * @return parser's name
     */
    public String getParserName() {
        if (parser == null) return "";
        return parser.getName();
    }

    /**
     * <p>
     * Returns name of the heuristic that was used in
     * creating this usecase
     * </p>
     * 
     * @return name of the heuristic
     */
    public String getHeuristicName() {
        if (heuristic == null) return "";
        return heuristic.getName();
    }

    /**
     * <p>
     * Returns the (mini) analyzemodel that was created from
     * this (and only this) usecase
     * </p>
     * 
     * @return mini analyzemodel
     */
    public AnalyzeModel getAnalyzeModel() {
        return model;
    }

    /**
     * <p>
     * Sets the (mini) analyzemodel that was created from
     * this (and only this) usecase
     * </p>
     * 
     * @param model
     *            this usecase was created from
     */
    public void setAnalyzeModel(AnalyzeModel model) {
        this.model = model;
    }

    /**
     * <p>
     * Mark this usecase as a sub usecase
     * </p>
     * 
     * @param sub
     *            true if this usecase is a sub-usecase
     */
    public void setAsSubUseCase(boolean sub) {
        subUseCase = sub;
    }

    /**
     * <p>
     * Is this usecase a sub-usecaes
     * </p>
     * 
     * @return true if it is
     */
    public boolean isSubUseCase() {
        return subUseCase;
    }

    /**
     * <p>
     * Is this usecase analyzed and added to main
     * analyzemodel
     * </p>
     * 
     * @return true if it is
     */
    public boolean isUseCaseAnalyzed() {
        return isUseCaseAnalyzed;
    }

    /**
     * <p>
     * Set wheiter this usecase is analyzed.
     * </p>
     * 
     * @param analyzed
     *            is the usecase analyzed
     */
    public void setUseCaseAnalyzed(boolean analyzed) {
        this.isUseCaseAnalyzed = analyzed;
    }

    /**
     * <p>
     * Mark this usecase as analyzed
     * </p>
     * 
     */
    public void setUseCaseAnalyzed() {
        setUseCaseAnalyzed(true);
    }

    /**
     * <p>
     * Iterator for the usecase steps
     * </p>
     */
    public Iterator<UseCaseStep> iterator() {
        return new Iterator<UseCaseStep>() {
            int index = 0;

            public boolean hasNext() {
                return index < getStepCount();
            }

            public UseCaseStep next() {
                return getStep(index++);
            }

            public void remove() {
                throw new UnsupportedOperationException();
            }

        };
    }

    /**
     * <p>
     * Set relation id for this usecase. Only used for
     * connecting UseCaseSteps to subusecases
     * </p>
     * 
     * @param relationId
     *            this usecase's id loaded from file
     */
    public void setId(String relationId) {
        useCaseId = relationId;
    }

    /**
     * <p>
     * Returns this usecase's id.
     * </p>
     * 
     * @return usecase id
     */
    public String getUseCaseId() {
        return useCaseId;
    }

    /**
     * <p>
     * Returns the count of steps this usecase has
     * </p>
     * 
     * @return step count
     */
    public int getStepCount() {
        return useCaseSteps.size();
    }

    /**
     * <p>
     * Returns step with given index
     * </p>
     * 
     * @param index
     *            index of the step
     * @return UseCaseStep or null if index was incorrect
     */
    public UseCaseStep getStep(int index) {
        if (index < 0 || index >= getStepCount()) return null;
        return useCaseSteps.get(index);
    }

    /**
     * <p>
     * Removes all usecase's steps
     * </p>
     * 
     */
    public void clear() {
        useCaseSteps.clear();
    }

    /**
     * <p>
     * Adds a new step to this usecase
     * </p>
     * 
     * @param step
     *            step's description
     * @param subUseCase
     *            reference to sub usecase
     */
    public void addStep(String step, UseCase subUseCase) {
        useCaseSteps.add(new UseCaseStep(step, subUseCase));
    }

    /**
     * <p>
     * Adds a new UseCaseStep to this usecase
     * </p>
     * 
     * @param step
     *            UseCaseStep object to add
     */
    public void addStep(UseCaseStep step) {
        if (step != null) useCaseSteps.add(step);
    }

    /**
     * <p>
     * Returns the url where this usecase was loaded from
     * </p>
     * 
     * @return url where this usecase was loaded from
     */
    public URL getUrl() {
        return url;
    }

    /**
     * <p>
     * Sets the url where this usecase was loaded from
     * </p>
     * 
     * @param url
     */
    public void setUrl(URL url) {
        this.url = url;
    }

    /**
     * <p>
     * Returns this usecase's name
     * </p>
     * 
     * @return name
     */
    public String getName() {
        return name;
    }

    /**
     * <p>
     * Sets a new name for this usecase
     * </p>
     * 
     * @param name
     *            new name
     */
    public void setName(String name) {
        if (name == null) return;
        this.name = name;
    }

    /**
     * <p>
     * Returns this usecase as a string with this format:
     * </p>
     * 
     * <pre><code>
     *   Name
     *   ---
     *   (step index). (step)
     *   (step index). (step)
     *   ...
     * </code></pre>
     */
    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append(name + "\n---\n");
        for (int i = 0; i < getStepCount(); i++) {
            sb.append(Integer.toString(i) + ". "
                    + getStep(i) + "\n");
        }
        return sb.toString();
    }

    /**
     * <p>
     * Equals method for usecases.
     * </p>
     */
    public boolean equals(Object obj) {
        if (obj != null && obj instanceof UseCase) {
            UseCase other = (UseCase) obj;

            // This could be checked easier this way, but it
            // isn't 100% reliable
            // return (other.hashCode() == this.hashCode());

            // Check that the both usecases have equal
            // amount of steps
            if (other.getStepCount() != getStepCount()) return false;

            // Check that the steps are equal, it is up to
            // step to check
            // wheither substeps are equal
            for (int i = 0; i < getStepCount(); i++) {
                if (!getStep(i).equals(other.getStep(i))) return false;
            }

            // Check that the names are equal
            if (!this.getName().equals(other.getName())) return false;
            return true;
        }

        return false;
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#hashCode()
     */
    @Override
    public int hashCode() {
        return toString().hashCode();
    }

}
