/*
 *  Copyright (C) 2006  UCOT
 *
 *  Distributed under the terms of the MIT License.
 *  See "LICENCE" or http://www.opensource.org/licenses/mit-license.php for details.
 */
package ucot.model;

/**
 * <p>
 * Error which is thrown if no entity of given name is
 * found.
 * </p>
 */
public class NoSuchParentException extends
        AnalyzeModelException {

    private static final long serialVersionUID = 1419331743786346560L;

    /** The name of the entity. */
    String entityName;

    /**
     * <p>
     * Builds message for the error. Message looks like
     * this: <br\>
     * 
     * <pre><code>
     *   No such entity called 'entityName'.
     * </code></pre>
     * 
     * </p>
     * 
     * @param entityName
     *            The entity name used in the error.
     * @return The error message string.
     */
    protected static String buildMessage(String entityName) {
        return "No such entity called '" + entityName
                + "'.";
    }

    /**
     * <p>
     * Constructs <code>NoSuchParentException</code> with
     * entity name that is missing.
     * </p>
     * 
     * @param entityName
     *            The name of the entity that is not found.
     */
    public NoSuchParentException(String entityName) {
        super(buildMessage(entityName));
        this.entityName = entityName;
    }

    /**
     * <p>
     * Constructs <code>NoSuchParentException</code> with
     * entity name that is missing and message.
     * </p>
     * 
     * @param entityName
     *            The name of the entity that is not found.
     * @param message
     *            The message.
     */
    public NoSuchParentException(String entityName,
            String message) {
        super(buildMessage(entityName) + " " + message);
        this.entityName = entityName;
    }

    /**
     * <p>
     * Constructs <code>NoSuchParentException</code> with
     * entity name that is missing and the cause.
     * </p>
     * 
     * @param entityName
     *            The name of the entity that is not found.
     * @param cause
     *            The cause.
     */
    public NoSuchParentException(String entityName,
            Throwable cause) {
        super(buildMessage(entityName), cause);
        this.entityName = entityName;
    }

    /**
     * <p>
     * Constructs <code>NoSuchParentException</code> with
     * entity name that is missing, message containing some
     * explanation and the cause.
     * </p>
     * 
     * @param entityName
     *            The name of the entity that is not found.
     * @param message
     *            The message.
     * @param cause
     *            The cause.
     */
    public NoSuchParentException(String entityName,
            String message, Throwable cause) {
        super(buildMessage(entityName) + " " + message,
                cause);
        this.entityName = entityName;
    }

}
