/*
 *  Copyright (C) 2006  UCOT
 *
 *  Distributed under the terms of the MIT License.
 *  See "LICENCE" or http://www.opensource.org/licenses/mit-license.php for details.
 */
package ucot.output;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.Vector;
import java.util.logging.Logger;

import net.sourceforge.gxl.GXLAttr;
import net.sourceforge.gxl.GXLBool;
import net.sourceforge.gxl.GXLDocument;
import net.sourceforge.gxl.GXLEdge;
import net.sourceforge.gxl.GXLGXL;
import net.sourceforge.gxl.GXLGraph;
import net.sourceforge.gxl.GXLNode;
import net.sourceforge.gxl.GXLString;
import net.sourceforge.gxl.GXLTup;
import ucot.ModuleProperties;
import ucot.exceptions.BadPropertyValueException;
import ucot.model.AnalyzeModel;
import ucot.model.AnalyzeModelException;
import ucot.model.ModelEditor;

/**
 * GXL output adapter
 * 
 * @author pajumasu
 * 
 */
public class GXLOutput extends ModuleProperties implements
        OutputInterface {

    public static final String name = "GXL";

    public static final String DEFAULT_FILE_EXTENSION = "gxl";

    public static final String ATTRIBUTE_TYPE = "type";

    public static final String ATTRIBUTE_NAME = "name";

    public static final String ATTRIBUTE_TYPE_NAME = "attribute";

    public static final String METHOD_TYPE_NAME = "method";

    public static final String PARENT_TYPE_NAME = "parent";

    @SuppressWarnings(value = { "unused" } // Suppreess
                                            // unused
                                            // warnings.
    )
    private Logger logger = Logger.getLogger("ucot.output");

    public void output(AnalyzeModel analyzeModel, URL url)
            throws IOException {
        // Create document and elements

        GXLDocument gxlDocument = new GXLDocument();
        GXLGraph graph = new GXLGraph("UCOT-Model");
        GXLGXL gxl = gxlDocument.getDocumentElement();
        gxl.setAttribute("xmlns:xlink",
                "http://www.w3.org/1999/xlink");
        gxl.add(graph);
        graph.setAttribute("edgemode", "directed");
        graph.setAttribute("hypergraph", "false");

        Map<String, GXLNode> nameToNodeMap = new HashMap<String, GXLNode>();

        ModelEditor editor = analyzeModel.getEditor();

        Set<String> entityNames = editor.getEntityNames();
        Vector<String> entityNamesOrdered = new Vector<String>();
        entityNamesOrdered.addAll(entityNames);
        Collections.sort(entityNamesOrdered);
        /*
         * URI containsURI = null; URI nodeURI = null;
         * 
         * try { //containsURI = new
         * URI("http://www.gupro.de/GXL/gxl-1.0.gxl#contains");
         * //nodeURI = new
         * URI("http://www.gupro.de/GXL/gxl-1.0.gxl#NodeClass"); }
         * catch (URISyntaxException e) {
         * logger.warning(e.getMessage());
         * logger.warning(StringTools.exceptionStackTrace(e)); }
         */

        // Create nodes
        int nodeCount = 0;
        for (String entity : entityNamesOrdered) {
            GXLNode node = new GXLNode(entity);
            // node.setType(nodeURI);
            graph.add(node);
            node.setID("node" + (++nodeCount));
            node.add(new GXLAttr("name", new GXLString(
                    entity)));
            String type = analyzeModel.getEditor()
                    .getEntityType(entity);
            if (type != null && type.length() > 0) {
                node.add(new GXLAttr("type", new GXLString(
                        type)));
            }
            nameToNodeMap.put(entity, node);

        }

        int edgecount = 0;

        // Create method edges.
        for (String entity : entityNames) {
            Set<String> methods = editor
                    .getMethodNames(entity);
            GXLNode node1 = nameToNodeMap.get(entity);
            for (String method : methods) {
                Set<String> targets = editor
                        .getEntitiesInfluencedByMethod(
                                entity, method);
                for (String target : targets) {
                    GXLNode node2 = nameToNodeMap
                            .get(target);
                    GXLEdge edge1 = new GXLEdge(node1,
                            node2);
                    edge1.setID("edge" + (++edgecount));
                    GXLString methodValue = new GXLString(
                            METHOD_TYPE_NAME);
                    edge1.setAttr(ATTRIBUTE_TYPE,
                            methodValue);
                    GXLString methodName = new GXLString(
                            method);
                    edge1.setAttr(ATTRIBUTE_NAME,
                            methodName);
                    edge1.setAttr("isdirected",
                            new GXLBool(true));
                    graph.add(edge1);
                }
            }
        }

        // Create attribute edges
        for (String entity : entityNames) {
            Set<String> attributes = editor
                    .getAttributeNames(entity);
            GXLNode node1 = nameToNodeMap.get(entity);
            for (String attribute : attributes) {
                GXLNode node2 = nameToNodeMap
                        .get(attribute);
                GXLEdge edge1 = new GXLEdge(node1, node2);
                edge1.setID("edge" + (++edgecount));
                GXLString attributeValue = new GXLString(
                        ATTRIBUTE_TYPE_NAME);
                edge1.setAttr(ATTRIBUTE_TYPE,
                        attributeValue);
                edge1.setAttr("isdirected", new GXLBool(
                        true));
                GXLTup tup = new GXLTup();
                try {
                    tup.add(new GXLString(editor
                            .getAttributeFromCardinal(
                                    entity, attribute)));
                    tup.add(new GXLString(editor
                            .getAttributeToCardinal(entity,
                                    attribute)));
                } catch (AnalyzeModelException e) {
                    e.printStackTrace();
                }
                edge1.setAttr("limits", tup);
                // edge1.setType(containsURI);
                graph.add(edge1);
            }
        }

        // Create parent edges
        for (String entity : entityNames) {
            Set<String> parents = editor.getParents(entity);
            GXLNode node1 = nameToNodeMap.get(entity);
            for (String parent : parents) {
                GXLNode node2 = nameToNodeMap.get(parent);
                GXLEdge edge1 = new GXLEdge(node1, node2);
                edge1.setID("edge" + (++edgecount));
                GXLString parentValue = new GXLString(
                        PARENT_TYPE_NAME);
                edge1.setAttr(ATTRIBUTE_TYPE, parentValue);
                edge1.setAttr("isdirected", new GXLBool(
                        true));
                graph.add(edge1);
            }
        }

        // Write the document to file
        File file = new File(url.getFile());
        gxlDocument.write(file);
    }

    public String getName() {
        return name;
    }

    /**
     * Test program used while programming this class
     * 
     * @param args
     */
    public static void main(String args[]) throws Throwable {
        File file = new File(
                "./examples/instance/gxl/attredges/attredges.gxl");
        InputStream in = new FileInputStream(file);
        GXLDocument gxlDocument = new GXLDocument(in);
        GXLGXL gxl = gxlDocument.getDocumentElement();
        System.out.println(gxl.getAttribute("xmlns:xlink"));
        for (int i = 0; i < gxl.getChildCount(); i++) {
            System.out.println(gxl.getChildAt(i));
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see ucot.ModulePropertyInterface#loadDefaultProperties()
     */
    public Properties loadDefaultProperties() {
        Properties result = super.loadDefaultProperties();
        // TODO Set default properties here, for example:
        // result.setProperty("SETTING_1", "Woot");
        // result.setProperty("SETTING_2", "WTF?");
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see ucot.ModulePropertyInterface#applyProperties()
     */
    public void applyProperties()
            throws BadPropertyValueException {
        // TODO Auto-generated method stub

    }

    /*
     * (non-Javadoc)
     * 
     * @see ucot.output.OutputInterface#getFileExtension()
     */
    public String getFileExtension() {
        return DEFAULT_FILE_EXTENSION;
    }
}
