/*
 *  Copyright (C) 2006  UCOT
 *
 *  Distributed under the terms of the MIT License.
 *  See "LICENCE" or http://www.opensource.org/licenses/mit-license.php for details.
 */
package ucot.test;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.Vector;

import junit.framework.Assert;

import org.junit.Test;

import ucot.core.Core;
import ucot.input.UseCase;
import ucot.input.UseCaseCollection;

/**
 * This class is implements all critical tests for the UCOT
 * core component.
 * 
 * @author tujupien
 */
public class CoreTest {
    Core core = new Core();

    URL testFile;

    final String testFileName = "Core-SimpleInputAdapter-UseCases-Test.txt";

    /**
     * Method for centralized UseCase loading. This method
     * loads use cases from the file pointed out by
     * testFileName. That file should include only one valid
     * UseCase and one invalid if that is possible.
     * 
     * @return Boolean true if no error occured and false if
     *         something happened.
     */
    private boolean loadUseCases() {
        // Don't load the file again if it's been loaded
        // already.
        if (core.getUseCaseCollection().getUseCaseCount() > 0) return true;

        // Initialize testFile URL variable only if it's not
        // yet been
        // initialized.
        if (testFile == null) {
            try {
                testFile = new URL("file", "", -1,
                        testFileName);
            } catch (MalformedURLException murle) {
                System.out
                        .println("TEST: Missing Test Data file: "
                                + testFileName);
                return false;
            }
        }

        // Load use cases.
        core.loadUseCases(testFile);
        return true;
    }

    /**
     * Test method for core shutdown and startup routines.
     */
    @Test
    public void testCoreShutdownAndStartup() {

        // ****************************************************************
        // NOTICE: Comment out the System.exit(0) from
        // core's shutdown
        // routine to enable all tests.
        // ****************************************************************

        System.out
                .println("TEST: Testing core component's startup and shutdown routines.");
        // core pointer variable should not be null at this
        // point.
        Assert.assertNotNull(core);
        // Test shutdown routine. Add some UseCases to the
        // UseCaseCollection
        // and random data to the AnalyzeModel.
        this.loadUseCases();
        try {
            UseCase uc = new UseCase();
            uc.setName("UselessCase");
            uc.addStep("Foo Bar Doo Gar", null);
            Vector<UseCase> useCases = new Vector<UseCase>();
            useCases.add(uc);
            core.addToAnalyzeModel(useCases);
        } catch (Exception e) {
            System.out
                    .println("TEST: Adding use cases to analyze model failed.");
            Assert.fail();
        }
        core.shutdown();
        // Shutdown routine should clear cores's the
        // UseCaseCollection and the
        // AnalyzeModel.
        // Assert.assertEquals(0,
        // core.getUseCaseCollection().getUseCaseCount());
        Assert.assertNull(core.getUseCaseCollection());
        core = null;
        core = new Core();
    }

    /**
     * Method for testing loading of use cases. Loading use
     * cases fro testFileName should result only one valid
     * use case in the core component's useCaseCollection.
     */
    @Test
    public void testUseCaseLoading() {
        // If use case loading fails, this test should fail
        // too.
        if (!loadUseCases()) Assert.fail();

        // Check use cases integrity.
        UseCaseCollection collection = core
                .getUseCaseCollection();
        System.out
                .println("TEST: Testing use case loading.\n"
                        + "TEST: Following use cases were found from test file <"
                        + testFileName + ">");
        for (int i = 0; i < collection.getUseCaseCount(); i++) {
            System.out.println("TEST: -- "
                    + collection.getUseCase(i).getName());
        }

        // There should be only one use case in the
        // collection.
        Assert
                .assertEquals(collection.getUseCaseCount(),
                        3);
    }
}
