/*
 *  Copyright (C) 2006  UCOT
 *
 *  Distributed under the terms of the MIT License.
 *  See "LICENCE" or http://www.opensource.org/licenses/mit-license.php for details.
 */
package ucot.ui;

import ucot.ModulePropertyInterface;
import ucot.core.ControlInterface;

/**
 * <p>
 * Interface for the UI. A means for the core component to
 * notify UI that certain actions have been taken. For
 * example that file has been loaded, use cases have been
 * loaded or the <code>ControlInterface</code> has been
 * changed.
 * </p>
 * 
 * <p>
 * In practice any user interface designed for UCOT program
 * should implement this interface because these methods are
 * used for the interclass communication and the core
 * component uses these methods to control the user
 * interface on some level and to inform user about certain
 * things happening on a lower level of the program.
 * </p>
 * 
 * @author vevijopi, tujupien
 * 
 */
public interface UIInterface extends
        ModulePropertyInterface {

    /**
     * <p>
     * Method for signaling the user interface that the
     * analyze model has been (successfully) exported.
     * </p>
     */
    public void exportDone();

    /**
     * <p>
     * Method for signaling the user interface that the
     * analyze model has been (successfully) loaded.
     * </p>
     */
    public void analyzeModelLoaded();

    /**
     * <p>
     * Method for signaling user interface that use cases
     * have been (successfully) loaded from file.
     * </p>
     */
    public void useCasesLoaded();

    /**
     * <p>
     * Core signals user interface that usecases have been
     * parsed, ran heuristic on and been added to given
     * analyze model.
     * </p>
     * 
     * @param foundEntities
     *            How many entities the parser found.
     * @param addedEntities
     *            How many entities were added.
     */
    public void useCaseAdded(int foundEntities,
            int addedEntities);

    /**
     * <p>
     * Set a new control interface for the user interface to
     * use.
     * </p>
     * 
     * @param a
     *            <code>ControlInterface</code> to the
     *            UCOT core.
     */
    public void setControlInterface(ControlInterface a);

    /**
     * <p>
     * Prints an error message to the screen.
     * </p>
     * 
     * @param errorMessage
     *            Description of the error.
     * @param errorTitle
     *            Title of the dialog.
     */
    public void printError(String errorMessage,
            String errorTitle);

    public void printError(String errorMessage);

    /**
     * <p>
     * Prints a warning to the screen.
     * </p>
     * 
     * @param warningMessage
     *            Description of the warning.
     * @param warningTitle
     *            Title of the dialog.
     */
    public void printWarning(String warningMessage,
            String warningTitle);

    public void printWarning(String warningMessage);

    /**
     * <p>
     * Method for getting a new progressbar for showing the
     * current progress status to the user and halting all
     * other usage of the model editor.
     * </p>
     * 
     * @return ProgressBar interface to the progress bar.
     */
    public ProgressBarInterface getProgressBar();
}
