/*
 *  Copyright (C) 2006  UCOT
 *
 *  Distributed under the terms of the MIT License.
 *  See "LICENCE" or http://www.opensource.org/licenses/mit-license.php for details.
 */
package ucot.ui.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;
import java.util.Properties;
import java.util.Set;
import java.util.Vector;
import java.util.logging.Logger;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JSplitPane;

import ucot.Messages;
import ucot.ModulePropertyInterface;
import ucot.core.ControlInterface;
import ucot.exceptions.BadPropertyValueException;
import ucot.input.UseCaseCollection;
import ucot.model.ModelEditor;
import ucot.output.OutputInterface;
import ucot.ui.ProgressBarInterface;
import ucot.ui.UIInterface;
import ucot.ui.UseCasePanelInterface;
import ucot.ui.gui.dialog.AboutDialog;
import ucot.ui.gui.dialog.EntityPropertiesDialog;
import ucot.ui.gui.dialog.ProgressBarDialog;
import ucot.ui.gui.dialog.SettingsDialog;
import ucot.ui.gui.dot.DotColorTheme;
import ucot.ui.gui.dot.DotPanel;
import ucot.ui.gui.dot.DotPanel.ExportImageType;
import ucot.ui.gui.tree.analyzemodeltree.AnalyzeModelTree;
import ucot.ui.gui.tree.analyzemodeltree.AnalyzeTreeModel;
import ucot.ui.gui.tree.usecasetree.UseCaseTree;
import ucot.utils.CustomFileFilter;
import ucot.utils.FileTools;
import ucot.utils.PropertiesTools;
import ucot.utils.StringTools;

/**
 * <p>
 * Simple graphical user interface for UCOT program which
 * uses dot (in DotPanel) to draw graphics. This class is
 * inherited from JFrame and implements the UIInterface
 * defined in ucot.ui package.
 * </p>
 * 
 * @author ilanliuk, tujupien, pajumasu
 */
public class GraphicalUI extends JFrame implements
        UIInterface, Observer, ModulePropertyInterface {

    private static final long serialVersionUID = -4246720431258449398L;

    private ControlInterface core = null;

    private ActionListener menuListener;

    private WindowListener windowListener;

    private AnalyzeModelTree analyzeModelTree;

    private JScrollPane modelTreeScrollPane;

    private UseCaseTree useCaseTree;

    private JScrollPane useCaseTreeScrollPane;

    private JMenuBar menu;

    private DotPanel dotPanel;

    private JScrollPane dotScrollPane;

    private JLabel dotTimer = new JLabel(" 0 ms ");

    private UseCasePanelInterface useCasePanel = new SimpleUseCasePanel();

    private JScrollPane useCasePanelScrollPane = new JScrollPane(
            (JPanel) useCasePanel);

    private Statusbar statusbar = new Statusbar();

    private EntityPropertiesDialog entityPropertiesDialog = new EntityPropertiesDialog(
            this);

    private ProgressBarDialog progressBarDialog;

    private URL currentFile;

    private boolean changesMade = false;

    private JLabel changesIndicator = new JLabel(" "
            + UNMODIFIED_STRING + " ");

    private Logger logger = Logger.getLogger("ucot.ui.gui");

    protected URL propertiesURL = getPropertiesURL();

    protected Properties properties = loadDefaultProperties();

    public Set<String> entityTypes;

    protected static String SERIALIZATION_DESCRIPTION = Messages
            .getString("GraphicalUI.SERIALIZATION_FILE_DESCRIPTION");

    protected static String[] SERIALIZATION_EXTENSIONS = { "ser" };

    protected static String PROGRAM_TITLE = "UCOT";

    protected final static String DEFAULT_ERROR_TITLE = Messages
            .getString("GraphicalUI.DEFAULT_ERROR_TITLE");

    protected final static String DEFAULT_QUESTION_TITLE = Messages
            .getString("GraphicalUI.DEFAULT_QUESTION_TITLE");

    protected final static String DEFAULT_WARNING_TITLE = Messages
            .getString("GraphicalUI.DEFAULT_WARNING_TITLE");

    public final static String EXPORT_ALL_GRAPH_FILENAME = "graph";

    public final static String EXPORT_ALL_DATA_FILENAME = "data";

    public final static String EXPORT_ALL_SERIALIZATION_FILENAME = "analyze_model";

    public final static String EXPORT_ALL_LOG_FILENAME = "modification_log";

    public final static String EXPORT_ALL_DOT_FILENAME = "graph.dot";

    // Property keys.
    public final static String PROPERTY_DOT_PATH = "DOT_PATH";

    public final static String PROPERTY_EPS_TO_PDF_PATH = "EPS_TO_PDF_PATH";

    public final static String PROPERTY_TEMP_INPUT_FILE = "TEMP_INPUT_FILE";

    public final static String PROPERTY_TEMP_OUTPUT_FILE = "TEMP_OUTPUT_FILE";

    public final static String PROPERTY_TEMP_EPS_FILE = "TEMP_EPS_FILE";

    public final static String PROPERTY_DOT_HIGHLIGHT_COLOR = "DOT_HIGHLIGHT_COLOR";

    public final static String PROPERTY_DOT_COLOR = "DOT_COLOR";

    public final static String PROPERTY_ENTITY_TYPES = "ENTITY_TYPES";

    public final static String PROPERTY_GRAPH_FONT_SIZE = "GRAPH_FONT_SIZE";

    protected static String CLEAR_MODEL_QUESTION_TITLE = Messages
            .getString("GraphicalUI.CLEAR_MODEL_QUESTION_TITLE");

    protected static String CLEAR_MODEL_QUESTION = Messages
            .getString("GraphicalUI.CLEAR_MODE_QUESTION");

    protected static String NEW_MODEL_QUESTION_TITLE = Messages
            .getString("GraphicalUI.NEW_MODEL_QUESTION_TITLE");

    protected static String NEW_MODEL_QUESTION = Messages
            .getString("GraphicalUI.NEW_MODEL_QUESTION");

    protected static String QUIT_QUESTION_TITLE = Messages
            .getString("GraphicalUI.QUIT_QUESTION_TITLE");

    protected static String QUIT_QUESTION = Messages
            .getString("GraphicalUI.QUIT_QUESTION");

    protected static String SAVE_BEFORE_QUIT_QUESTION_TITLE = Messages
            .getString("GraphicalUI.SAVE_BEFORE_QUIT_QUESTION_TITLE");

    protected static String SAVE_BEFORE_QUIT_QUESTION = Messages
            .getString("GraphicalUI.SAVE_BEFORE_QUIT_QUESTION");

    protected static String LOG_FILE_LOAD_TRIGGERED = Messages
            .getString("GraphicalUI.LOG_FILE_LOAD_TRIGGERED");

    protected static String LOG_QUIT_PROGRAM_TRIGGERED = Messages
            .getString("GraphicalUI.LOG_QUIT_PROGRAM_TRIGGERED");

    protected static String LOG_QUIT_PROGRAM = Messages
            .getString("GraphicalUI.LOG_QUIT_PROGRAM");

    protected static String LOG_CLEAR_MODEL = Messages
            .getString("GraphicalUI.LOG_CLEAR_MODEL");

    protected static String LOG_SETTINGS_TRIGGERED = Messages
            .getString("GraphicalUI.LOG_SETTINGS_TRIGGERED");

    protected static String LOG_EXPORT_AS_IMAGE_TRIGGERED = Messages
            .getString("GraphicalUI.LOG_EXPORT_AS_IMAGE_TRIGGERED");

    protected static String LOG_EXPORT_DONE_MESSAGE = Messages
            .getString("GraphicalUI.LOG_EXPORT_DONE_MESSAGE");

    protected static String LOG_ANALYZE_MODEL_LOADED = Messages
            .getString("GraphicalUI.LOG_ANALYZE_MODEL_LOADED");

    protected static String LOG_USE_CASES_LOADED = Messages
            .getString("GraphicalUI.LOG_USE_CASES_LOADED");

    protected static String LOG_USE_CASE_LOADING_EXCEPTION = Messages
            .getString("GraphicalUI.LOG_USE_CASE_LOADING_EXCEPTION");

    protected static String LOG_USE_CASES_ADDED = Messages
            .getString("GraphicalUI.LOG_USE_CASES_ADDED");

    protected static String FILE_MENU_CAPTION = Messages
            .getString("GraphicalUI.FILE_MENU_CAPTION");

    protected static String NEW_MENU_CAPTION = Messages
            .getString("GraphicalUI.NEW_MENU_CAPTION");

    protected static String OPEN_MENU_CAPTION = Messages
            .getString("GraphicalUI.OPEN_MENU_CAPTION")
            + "...";

    protected static String LOAD_MENU_CAPTION = Messages
            .getString("GraphicalUI.LOAD_MENU_CAPTION")
            + "...";

    protected static String SAVE_MENU_CAPTION = Messages
            .getString("GraphicalUI.SAVE_MENU_CAPTION");

    protected static String MODIFICATION_LOG_MENU_CAPTION = Messages
            .getString("GraphicalUI.MODIFICATION_LOG_MENU_CAPTION")
            + "...";

    protected static String SAVE_AS_MENU_CAPTION = Messages
            .getString("GraphicalUI.SAVE_AS_MENU_CAPTION")
            + "...";

    protected static String EXPORT_MENU_CAPTION = Messages
            .getString("GraphicalUI.EXPORT_MENU_CAPTION")
            + "...";

    protected static String EXPORT_AS_IMAGE_MENU_CAPTION = Messages
            .getString("GraphicalUI.EXPORT_AS_IMAGE_MENU_CAPTION")
            + "...";

    protected static String EXPORT_ALL_MENU_CAPTION = Messages
            .getString("GraphicalUI.EXPORT_ALL_MENU_CAPTION")
            + "...";

    protected static String CLEAR_MODEL_MENU_CAPTION = Messages
            .getString("GraphicalUI.CLEAR_MODEL_MENU_CAPTION")
            + "...";

    protected static String QUIT_MENU_CAPTION = Messages
            .getString("GraphicalUI.QUIT_MENU_CAPTION");

    protected static String PROGRAM_MENU_CAPTION = Messages
            .getString("GraphicalUI.PROGRAM_MENU_CAPTION");

    protected static String SETTINGS_MENU_CAPTION = Messages
            .getString("GraphicalUI.SETTINGS_MENU_CAPTION")
            + "...";

    protected static String DOT_USE_HORIZONGAL_LAYOUTING_CAPTION = Messages
            .getString("GraphicalUI.DOT_USE_HORIZONTAL_LAYOUTING_CAPTION");

    protected static String HELP_MENU_CAPTION = Messages
            .getString("GraphicalUI.HELP_MENU_CAPTION");

    protected static String ABOUT_MENU_CAPTION = Messages
            .getString("GraphicalUI.ABOUT_MENU_CAPTION");

    protected static String PROGRESS_BAR_DIALOG_TITLE = Messages
            .getString("GraphicalUI.PROGRESS_BAR_DIALOG_TITLE");

    protected static String MODIFIED_STRING = Messages
            .getString("GraphicalUI.MODIFIED_STRING");

    protected static String UNMODIFIED_STRING = Messages
            .getString("GraphicalUI.UNMODIFIED_STRING");

    protected static String NEW_FILE_MENU_ACTION = "NEW_MODEL";

    protected static String MODIFICATION_LOG_MENU_ACTION = "MODIFICATION_LOG";

    protected static String OPEN_FILE_MENU_ACTION = "OPEN_FILE";

    protected static String LOAD_FILE_MENU_ACTION = "LOAD_FILE";

    protected static String SAVE_FILE_MENU_ACTION = "SAVE_FILE";

    protected static String SAVE_AS_FILE_MENU_ACTION = "SAVE_FILE_AS";

    protected static String EXPORT_MENU_ACTION = "EXPORT";

    protected static String EXPORT_ALL_MENU_ACTION = "EXPORT_ALL";

    protected static String QUIT_MENU_ACTION = "QUIT_PROGRAM";

    protected static String CLEAR_MODEL_MENU_ACTION = "CLEAR_MODEL";

    protected static String EXPORT_AS_IMAGE_MENU_ACTION = "EXPORT_AS_IMAGE";

    protected static String SETTINGS_MENU_ACTION = "PROGRAM_SETTINGS";

    protected static String CHANGE_DOT_LAYOUTING_ACTION = "CHANGE_DOT_LAYOUTING";

    protected static String ABOUT_MENU_ACTION = "ABOUT_UCOT";

    /**
     * <p>
     * Method for changing the changes made status.
     * Practically anything that modifies the analyze model
     * should call this method with parameter true, and only
     * the save and new operations should call this method
     * with parameter false.
     * </p>
     * 
     * @param changesMade
     *            New status which indicates whether or not
     *            any changes have been made to the current
     *            analyze model since last save operation..
     */
    public void setChanged(boolean changesMade) {
        this.changesMade = changesMade;
        if (changesMade) changesIndicator.setText(" "
                + MODIFIED_STRING + " ");
        else changesIndicator.setText(" "
                + UNMODIFIED_STRING + " ");
    }

    /**
     * <p>
     * Action performed: Quit is chosen from the File menu.
     * </p>
     * 
     * @param quitWithoutPrompting
     *            Indicates wether the user really wants to
     *            quit without prompting another question to
     *            confirm that.
     */
    private void menuQuit(boolean quitWithoutPrompting) {
        if (changesMade) {
            if (!quitWithoutPrompting
                    && GUIUtils.questionDialog(this,
                            QUIT_QUESTION,
                            QUIT_QUESTION_TITLE) != (JOptionPane.YES_OPTION)) return;
            int saveChanges = GUIUtils.questionDialog(this,
                    SAVE_BEFORE_QUIT_QUESTION,
                    SAVE_BEFORE_QUIT_QUESTION_TITLE);
            if (saveChanges == JOptionPane.YES_OPTION) {
                if (!saveModel(currentFile)) {
                    // No save after all this work and
                    // promises.
                    // Screw u guys I am not going to home
                    // (= quit from
                    // quitting).
                    return;
                }
            }
        }
        logger.info(LOG_QUIT_PROGRAM_TRIGGERED);
        logger.info(LOG_QUIT_PROGRAM);
        core.shutdown();
        // TODO Change to match "real shutdown routine"?
    }

    /**
     * <p>
     * Action performed: User clicks the export all menu
     * item.
     * </p>
     */
    private void menuExportAll() {

        ChoosedFile choosed = GUIUtils.showSaveFileDialog(
                this, null, JFileChooser.DIRECTORIES_ONLY);
        URL target = choosed.url;
        if (target == null) return;

        String path = target.getPath();

        // Save dot file.
        try {
            FileTools
                    .copyFile(
                            new File(
                                    getProperties()
                                            .getProperty(
                                                    GraphicalUI.PROPERTY_TEMP_INPUT_FILE)),
                            new File(path,
                                    EXPORT_ALL_DOT_FILENAME));
        } catch (IOException e) {
            logger.warning(StringTools
                    .exceptionStackTrace(e));
        }

        // Export graph image to the export directory.
        Vector<String> imageTypes = new Vector<String>();
        for (DotPanel.ExportImageType imageType : DotPanel.ExportImageType
                .values()) {
            imageTypes.add(imageType.toString());
            // Comment the following to disable exporting in
            // all image formats.
            this.dotPanel.exportImage(new File(path
                    + EXPORT_ALL_GRAPH_FILENAME + "."
                    + imageType.toString().toLowerCase()),
                    imageType);
        }

        // Remove comments to ask for just one image format
        // for the graph.
        // String imageType =
        // GUIUtils.showComboBoxOptionDialog("Select image
        // format", imageTypes);
        // this.dotPanel.exportImage(new File(path +
        // EXPORT_ALL_GRAPH_FILENAME +
        // "." +
        // imageType.toLowerCase()),
        // DotPanel.ExportImageType.valueOf(imageType));

        Vector<String> exportTypes = new Vector<String>();
        Vector<OutputInterface> outputInterfaces = core
                .getOutputs();
        for (OutputInterface outputInterface : outputInterfaces) {
            exportTypes.add(outputInterface.getName());
        }

        // Export model to selected format.
        // Uncomment the following line if you want to
        // export only one data
        // type.
        // String exportType =
        // GUIUtils.showComboBoxOptionDialog("Select export
        // format", exportTypes);
        URL exportURL = null;

        for (OutputInterface outputInterface : outputInterfaces) {
            // Uncomment the if block if you want to export
            // only one datatype.
            // if (
            // outputInterface.getName().equals(exportType)
            // ) {
            File exportFile = new File(path
                    + EXPORT_ALL_DATA_FILENAME + "."
                    + outputInterface.getFileExtension());
            try {
                exportURL = exportFile.toURL();
            } catch (Exception e) {
                logger.warning(StringTools
                        .exceptionStackTrace(e));
            }

            try {
                core.output(exportURL, outputInterface,
                        core.getAnalyzeModel());
            } catch (Exception e) {
                logger.warning(StringTools
                        .exceptionStackTrace(e));
            }
            break;
            // }
        }

        // Copy used use case files to the export directory.
        Set<URL> useCaseURLs = new HashSet<URL>();
        UseCaseCollection useCaseCollection = core
                .getUseCaseCollection();
        for (int i = 0; i < useCaseCollection
                .getUseCaseCount(); i++) {
            URL useCaseFile = useCaseCollection.getUseCase(
                    i).getUrl();
            useCaseURLs.add(useCaseFile);
        }

        for (URL useCaseFile : useCaseURLs) {
            try {
                FileTools.copyFile(useCaseFile, target);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

        // Serialize analyze model.
        URL serializationURL = null;
        File serializationFile = new File(path
                + EXPORT_ALL_SERIALIZATION_FILENAME + "."
                + SERIALIZATION_EXTENSIONS[0].toLowerCase());
        try {
            serializationURL = serializationFile.toURL();
        } catch (Exception e) {
            logger.warning(StringTools
                    .exceptionStackTrace(e));
        }

        core.saveAnalyzeModel(serializationURL);

        // Save modification log.
        try {
            core
                    .getAnalyzeModel()
                    .getEditor()
                    .saveUpdationsToFile(
                            new File(path,
                                    EXPORT_ALL_LOG_FILENAME));
        } catch (IOException e) {
            logger.warning(StringTools
                    .exceptionStackTrace(e));
        }

        // TODO Check for overwriting stuff before doing
        // it...
    }

    /**
     * <p>
     * Action performed: Clear model action is chosen from
     * the File menu.
     * </p>
     * 
     * @param clearWithoutPrompting
     */
    private void menuClearModel(
            boolean clearWithoutPrompting) {
        logger.info(LOG_CLEAR_MODEL);
        if (CLEAR_MODEL_QUESTION_TITLE.length() <= 0) {
            CLEAR_MODEL_QUESTION_TITLE = DEFAULT_QUESTION_TITLE;
        }
        if (clearWithoutPrompting
                || GUIUtils.questionDialog(this,
                        CLEAR_MODEL_QUESTION_TITLE,
                        CLEAR_MODEL_QUESTION) == JOptionPane.YES_OPTION) {
            core.clearAnalyzeModel();
            setChanged(true);
        }
    }

    /**
     * <p>
     * Action performed: User selects the new model option
     * from the menu.
     * </p>
     */
    private void menuNewModel() {
        if (GUIUtils.questionDialog(this,
                NEW_MODEL_QUESTION,
                NEW_MODEL_QUESTION_TITLE) == JOptionPane.YES_OPTION) {
            if (!saveModel(currentFile)) {
                // Bail out if model was not saved dispite
                // of beautiful
                // promises.
                return;
            }
        }
        currentFile = null;
        menuClearModel(true);
        setChanged(false);
        useCaseTree.clear();
    }

    /**
     * <p>
     * Action performed: Open file is chosen from the File
     * menu.
     * </p>
     */
    private void menuLoadUseCaseFile() {
        logger.info(LOG_FILE_LOAD_TRIGGERED);

        // TODO Add support for file filters.
        URL file = GUIUtils.showOpenFileDialog(this, null);

        if (null == file) return; // Return if no
                                    // selection.

        // If file was selected then load it.
        boolean changed = changesMade;
        core.loadUseCases(file);
        dotPanel.updateModel(core.getAnalyzeModel());
        setChanged(changed);
    }

    /**
     * <p>
     * Action performed: Export is chosen from the File
     * menu.
     * </p>
     */
    private void menuExport() {
        List<OutputInterface> outputInterfaces = core
                .getOutputs();
        Vector<CustomFileFilter> filters = new Vector<CustomFileFilter>();
        Map<String, OutputInterface> extensionToOutputterMap = new HashMap<String, OutputInterface>();
        for (OutputInterface oi : outputInterfaces) {
            CustomFileFilter filter = new CustomFileFilter(
                    oi.getName(), new String[] { oi
                            .getFileExtension() });
            filters.add(filter);
            extensionToOutputterMap.put(oi
                    .getFileExtension(), oi);
        }

        ChoosedFile choosed = GUIUtils.showSaveFileDialog(
                this, filters);
        URL file = choosed.url;
        if (file == null) return;

        OutputInterface oi = extensionToOutputterMap
                .get(choosed.filefilter.getExtensions()
                        .get(0));
        ;
        try {
            core.output(file, oi, core.getAnalyzeModel());
        } catch (Exception e) {
            logger.warning(StringTools
                    .exceptionStackTrace(e));
        }
    }

    /**
     * <p>
     * Action performed: Export is chosen from the File
     * menu.
     * </p>
     */
    private void menuModificationLog() {
        new ModificationLogWindow(core.getAnalyzeModel());
    }

    /**
     * <p>
     * Action performed: User wants to change program
     * settings.
     * </p>
     */
    private void menuSettings() {
        logger.info(LOG_SETTINGS_TRIGGERED);

        // Launch settings dialog
        SettingsDialog.showDialog(this);

        // TODO add dot panel to redraw here in case ok was
        // pressed
        // Also inform core to apply properties
    }

    /**
     * <p>
     * Action performed: User wants to save picture of the
     * model.
     * </p>
     */
    private void menuExportAsImage() {

        // TODO Prompt user for overwriting existing
        // files... same goes for GXL
        // over there ^^

        Vector<CustomFileFilter> filters = new Vector<CustomFileFilter>();
        for (ExportImageType imagetype : ExportImageType
                .values()) {
            filters.add(new CustomFileFilter(imagetype
                    .toString()
                    + " files", new String[] { imagetype
                    .toString().toLowerCase() }));
        }

        ChoosedFile choosed = GUIUtils.showSaveFileDialog(
                this, filters);
        URL file = choosed.url;
        if (file == null) return;

        try {
            File exportFile = new File(file.toURI());
            dotPanel.exportImage(exportFile);
        } catch (Exception e) {
            logger.warning(StringTools
                    .exceptionStackTrace(e));
        }
    }

    /**
     * <p>
     * Action performed: User changes the status of the
     * horizontal layouting checkbox.
     * </p>
     */
    private void changeDotLayouting() {
        dotPanel.setHorizontalLayout(!dotPanel
                .getHorizontalLayout());
    }

    /**
     * <p>
     * This method creates and initializes all required
     * window listeners for this graphical user interface.
     * </p>
     */
    private void createWindowListener() {
        /**
         * WindowListener for all window events for the GUI
         * window.
         */
        windowListener = new WindowListener() {
            /**
             * <p>
             * User is trying to close the window forcefully
             * from outside the main menu.
             * </p>
             */
            public void windowClosing(WindowEvent e) {
                logger.info(LOG_QUIT_PROGRAM_TRIGGERED);
                menuQuit(true);
            }

            public void windowActivated(WindowEvent e) {
            }

            public void windowClosed(WindowEvent e) {
            }

            public void windowDeactivated(WindowEvent e) {
            }

            public void windowDeiconified(WindowEvent e) {
            }

            public void windowIconified(WindowEvent e) {
            }

            public void windowOpened(WindowEvent e) {
            }

        };
    }

    /**
     * <p>
     * This method creates and initializes the menu listener
     * for the program menu bar.
     * </p>
     */
    private void createMenuListener() {
        /**
         * ActionListener for all button events within the
         * dialog.
         */
        menuListener = new ActionListener() {

            /**
             * <p>
             * This method is called when an action is
             * performed and it calls the corresponding
             * methods to handle the action.
             * </p>
             */
            public void actionPerformed(ActionEvent e) {
                if (LOAD_FILE_MENU_ACTION.equals(e
                        .getActionCommand())) {
                    menuLoadUseCaseFile();
                    return;
                }
                if (EXPORT_MENU_ACTION.equals(e
                        .getActionCommand())) {
                    menuExport();
                    return;
                }
                if (QUIT_MENU_ACTION.equals(e
                        .getActionCommand())) {
                    menuQuit(true);
                    return;
                }
                if (CLEAR_MODEL_MENU_ACTION.equals(e
                        .getActionCommand())) {
                    menuClearModel(false);
                    return;
                }
                if (SETTINGS_MENU_ACTION.equals(e
                        .getActionCommand())) {
                    menuSettings();
                    return;
                }
                if (EXPORT_AS_IMAGE_MENU_ACTION.equals(e
                        .getActionCommand())) {
                    menuExportAsImage();
                    return;
                }
                if (CHANGE_DOT_LAYOUTING_ACTION.equals(e
                        .getActionCommand())) {
                    changeDotLayouting();
                    return;
                }
                if (ABOUT_MENU_ACTION.equals(e
                        .getActionCommand())) {
                    showAboutDialog();
                    return;
                }
                if (SAVE_FILE_MENU_ACTION.equals(e
                        .getActionCommand())) {
                    saveModel(currentFile);
                    return;
                }
                if (SAVE_AS_FILE_MENU_ACTION.equals(e
                        .getActionCommand())) {
                    saveModel(null);
                    return;
                }
                if (NEW_FILE_MENU_ACTION.equals(e
                        .getActionCommand())) {
                    menuNewModel();
                    return;
                }
                if (MODIFICATION_LOG_MENU_ACTION.equals(e
                        .getActionCommand())) {
                    menuModificationLog();
                }
                if (OPEN_FILE_MENU_ACTION.equals(e
                        .getActionCommand())) {
                    loadModel();
                    return;
                }
                if (EXPORT_ALL_MENU_ACTION.equals(e
                        .getActionCommand())) {
                    menuExportAll();
                    return;
                }
            }
        };
    }

    /**
     * <p>
     * Constuctor for Graphical UI. This constructor sets up
     * the whole user interface and spawns it to the screen.
     * </p>
     */
    public GraphicalUI(ControlInterface controlInterface) {
        // Initialize control interface.
        this.core = controlInterface;
        core.getAnalyzeModel().addObserver(this);

        try {
            loadProperties();
        } catch (IOException e) {
            logger.warning(StringTools
                    .exceptionStackTrace(e));
        }

        // TODO parse entity types!
        // parse entity types from properties
        this.entityTypes = parseEntityTypes();
        this.setPreferredSize(new Dimension(1000, 700));

        // Initialize all panels.
        dotPanel = new DotPanel(this);
        dotScrollPane = new JScrollPane(dotPanel);
        dotScrollPane
                .setMinimumSize(new Dimension(300, 250));
        dotScrollPane.getHorizontalScrollBar()
                .setUnitIncrement(DotPanel.SCROLL_SPEED);
        dotScrollPane.getVerticalScrollBar()
                .setUnitIncrement(DotPanel.SCROLL_SPEED);
        this.setLayout(new BorderLayout());

        // create the tree form AnalyzeModel
        analyzeModelTree = new AnalyzeModelTree(this);
        modelTreeScrollPane = new JScrollPane(
                analyzeModelTree);
        modelTreeScrollPane.setMinimumSize(new Dimension(
                200, 150));

        createMenuListener();
        createWindowListener();
        this.addWindowListener(windowListener);

        useCasePanelScrollPane
                .setMinimumSize(new Dimension(300, 50));

        // Finalize analyze model tree initialization.
        analyzeModelTree.createMouseListener(core,
                entityPropertiesDialog);
        analyzeModelTree.setStructurePanel(dotPanel);

        // Create new use case tree.
        this.useCaseTree = new UseCaseTree(this.core);
        this.useCaseTree.setUseCasePanel(this.useCasePanel);
        this.useCaseTree.setDotPanel(this.dotPanel);

        // Put UseCaseTree into a ScrollPane.
        useCaseTreeScrollPane = new JScrollPane(
                this.useCaseTree);
        useCaseTreeScrollPane.setMinimumSize(new Dimension(
                200, 150));

        // Set default exit policy.
        this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        // Divide screen into four section with SplitPanes.
        JSplitPane treePane = new JSplitPane(
                JSplitPane.VERTICAL_SPLIT,
                modelTreeScrollPane, useCaseTreeScrollPane);
        JSplitPane viewPane = new JSplitPane(
                JSplitPane.VERTICAL_SPLIT, dotScrollPane,
                (JComponent) useCasePanelScrollPane);
        JSplitPane sp = new JSplitPane(
                JSplitPane.HORIZONTAL_SPLIT, treePane,
                viewPane);

        // Finalize initialization and make window visible.
        this.getContentPane().add(statusbar,
                BorderLayout.SOUTH);
        this.getContentPane().add(sp, BorderLayout.CENTER);
        this.menu = createMenu();
        this.setJMenuBar(menu);
        this.setTitle(PROGRAM_TITLE);

        // Modification status indicator.
        statusbar.addColumn(changesIndicator);

        // Configure statusbar.
        statusbar.addColumn(dotTimer);
        dotPanel.setTimerComponent(dotTimer);

        this.pack();
        treePane.setDividerLocation(0.7);
        viewPane.setDividerLocation(0.7);
        sp.setDividerLocation(0.3);
        this.setVisible(true);
    }

    /**
     * <p>
     * Method for spawning the about UCOT dialog.
     * </p>
     */
    private void showAboutDialog() {
        AboutDialog dialog = new AboutDialog(this);
        GUIUtils.centerDialog(dialog);
        dialog.setVisible(true);
    }

    /**
     * <p>
     * Method for creating a menu bar to the GUI.
     * </p>
     * 
     * @return Program menu bar as a JMenuBar object.
     */
    private JMenuBar createMenu() {

        /*
         * Menu will look something like this:
         * +------+----------+-------- | File | Program |
         * =========================== | Open | Settings |
         * +------+----------+ | Exit | +------+
         */

        JMenuBar result = new JMenuBar();

        // FILE menu.
        JMenu menu = new JMenu(FILE_MENU_CAPTION);
        JMenuItem menuItem;

        menuItem = new JMenuItem(NEW_MENU_CAPTION);
        menuItem.addActionListener(menuListener);
        menuItem.setActionCommand(NEW_FILE_MENU_ACTION);
        menu.add(menuItem);
        menuItem = new JMenuItem(OPEN_MENU_CAPTION);
        menuItem.addActionListener(menuListener);
        menuItem.setActionCommand(OPEN_FILE_MENU_ACTION);
        menu.add(menuItem);
        menuItem = new JMenuItem(LOAD_MENU_CAPTION);
        menuItem.addActionListener(menuListener);
        menuItem.setActionCommand(LOAD_FILE_MENU_ACTION);
        menu.add(menuItem);
        menu.add(new JSeparator());
        menuItem = new JMenuItem(SAVE_MENU_CAPTION);
        menuItem.addActionListener(menuListener);
        menuItem.setActionCommand(SAVE_FILE_MENU_ACTION);
        menu.add(menuItem);
        menuItem = new JMenuItem(SAVE_AS_MENU_CAPTION);
        menuItem.addActionListener(menuListener);
        menuItem.setActionCommand(SAVE_AS_FILE_MENU_ACTION);
        menu.add(menuItem);
        menuItem = new JMenuItem(
                EXPORT_AS_IMAGE_MENU_CAPTION);
        menuItem.addActionListener(menuListener);
        menuItem
                .setActionCommand(EXPORT_AS_IMAGE_MENU_ACTION);
        menu.add(menuItem);
        menuItem = new JMenuItem(EXPORT_MENU_CAPTION);
        menuItem.addActionListener(menuListener);
        menuItem.setActionCommand(EXPORT_MENU_ACTION);
        menu.add(menuItem);
        menuItem = new JMenuItem(EXPORT_ALL_MENU_CAPTION);
        menuItem.addActionListener(menuListener);
        menuItem.setActionCommand(EXPORT_ALL_MENU_ACTION);
        menu.add(menuItem);
        menu.add(new JSeparator());
        menuItem = new JMenuItem(CLEAR_MODEL_MENU_CAPTION);
        menuItem.addActionListener(menuListener);
        menuItem.setActionCommand(CLEAR_MODEL_MENU_ACTION);
        menu.add(menuItem);
        menu.add(new JSeparator());
        menuItem = new JMenuItem(
                MODIFICATION_LOG_MENU_CAPTION);
        menuItem.addActionListener(menuListener);
        menuItem
                .setActionCommand(MODIFICATION_LOG_MENU_ACTION);
        menu.add(menuItem);
        menuItem = new JMenuItem(QUIT_MENU_CAPTION);
        menuItem.addActionListener(menuListener);
        menuItem.setActionCommand(QUIT_MENU_ACTION);
        menu.add(menuItem);
        result.add(menu);

        // PROGRAM menu.
        menu = new JMenu(PROGRAM_MENU_CAPTION);

        menuItem = new JCheckBoxMenuItem(
                DOT_USE_HORIZONGAL_LAYOUTING_CAPTION);
        ((JCheckBoxMenuItem) menuItem).setSelected(dotPanel
                .getHorizontalLayout());
        menuItem.addActionListener(menuListener);
        menuItem
                .setActionCommand(CHANGE_DOT_LAYOUTING_ACTION);
        menu.add(menuItem);
        menu.add(new JSeparator());
        menuItem = new JMenuItem(SETTINGS_MENU_CAPTION);
        menuItem.addActionListener(menuListener);
        menuItem.setActionCommand(SETTINGS_MENU_ACTION);
        menu.add(menuItem);
        result.add(menu);

        // HELP menu.
        menu = new JMenu(HELP_MENU_CAPTION);

        menuItem = new JMenuItem(ABOUT_MENU_CAPTION);
        menuItem.addActionListener(menuListener);
        menuItem.setActionCommand(ABOUT_MENU_ACTION);
        menu.add(menuItem);
        result.add(menu);

        // Menu bar successfully created!
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see ucot.ui.UIInterface#exportDone()
     */
    public void exportDone() {
        logger.info(LOG_EXPORT_DONE_MESSAGE);
    }

    /*
     * (non-Javadoc)
     * 
     * @see ucot.ui.UIInterface#analyzeModelLoaded()
     */
    public void analyzeModelLoaded() {
        setChanged(true);
        ((AnalyzeTreeModel) analyzeModelTree.getModel())
                .updateAnalyzeModel();
        // analyzeModelTree.setModel(new
        // AnalyzeTreeModel(this));
        dotPanel.updateModel(core.getAnalyzeModel());
        logger.info(LOG_ANALYZE_MODEL_LOADED);

        setChanged(false);
    }

    /*
     * (non-Javadoc)
     * 
     * @see ucot.ui.UIInterface#useCasesLoaded()
     */
    public void useCasesLoaded() {
        setChanged(true);
        logger.info(LOG_USE_CASES_LOADED);
        UseCaseCollection collection = core
                .getUseCaseCollection();
        try {
            if (collection.getUseCaseCount() > 0) {
                // Ask user which use case he or she desires
                // to load.
                // UseCase uc =
                // useCaseSelector.selectUseCase(collection);
                // core.addToAnalyzeModel(uc);

            } else {
                throw new Exception(
                        LOG_USE_CASE_LOADING_EXCEPTION);
            }
        } catch (Exception e) {
            logger.warning(StringTools
                    .exceptionStackTrace(e));
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see ucot.ui.UIInterface#useCaseAdded(int, int)
     */
    public void useCaseAdded(int foundEntities,
            int addedEntities) {
        setChanged(true);
        GUIUtils
                .printInfo(
                        this,
                        Messages
                                .getFormatedMessage(
                                        "GraphicalUI.USE_CASE_ADDED",
                                        foundEntities,
                                        addedEntities),
                        Messages
                                .getString("GraphicalUI.USE_CASE_ADDED_HEADER"));
        useCasePanel.refresh();

        updateTypes();
        logger.info(LOG_USE_CASES_ADDED);
    }

    /*
     * (non-Javadoc)
     * 
     * @see ucot.ui.UIInterface#setControlInterface(ucot.core.ControlInterface)
     */
    public void setControlInterface(ControlInterface a) {
        core = a;
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.util.Observer#update(java.util.Observable,
     *      java.lang.Object)
     */
    public void update(Observable o, Object arg) {
        setChanged(true);
    }

    /**
     * <p>
     * This method serializes the current status of the
     * project.
     * </p>
     * 
     * @param target
     *            Path to the file used for saving. If this
     *            is null, then a save file dialog will be
     *            spawned.
     * @return true if model was saved, false otherwhise.
     */
    public boolean saveModel(URL target) {
        if (target == null) {
            Vector<CustomFileFilter> fileFilters = new Vector<CustomFileFilter>();
            fileFilters.add(new CustomFileFilter(
                    SERIALIZATION_DESCRIPTION,
                    SERIALIZATION_EXTENSIONS));
            // Filename was not given so we need to ask the
            // user about it.
            target = GUIUtils.showSaveFileDialog(this,
                    fileFilters).url;
            if (target == null) return false;
            // Now check wether the file already exists.
            File file = new File(target.getFile());
            // File exists, so we need to confirm the
            // overwriting of it.
            if (file.exists()) {
                int overwrite = GUIUtils
                        .questionDialog(
                                this,
                                Messages
                                        .getFormatedMessage(
                                                Messages
                                                        .getString("FILE_EXISTS_REPLACE_QUESTION"),
                                                file));
                if (overwrite == JOptionPane.NO_OPTION) return false;
            }
        }

        setChanged(false);

        core.saveAnalyzeModel(target);
        currentFile = target;
        return true;
    }

    /**
     * <p>
     * This function parses the entity types from the
     * property value and splits the separate types into a
     * String vector.
     * </p>
     * 
     * @return Vector containing the available entity types.
     */
    private Set<String> parseEntityTypes() {

        Set<String> result = new HashSet<String>();
        String typeString = this.properties
                .getProperty(PROPERTY_ENTITY_TYPES);

        // If there was no types in properties no need for
        // parsing
        if (!"".equals(typeString)) {

            String[] types = typeString.split(", ");

            for (int i = 0; i < types.length; i++) {
                result.add(types[i]);
            }

        }

        return result;
    }

    /**
     * <p>
     * Default save method. The previous file will be
     * overwritten.
     * </p>
     */
    public void saveModel() {
        saveModel(currentFile);
    }

    /**
     * <p>
     * Default method for loading an analyze model. The
     * serialization file is defined by the user who gets an
     * file open dialog on his face before loading takes
     * place.
     * </p>
     */
    public void loadModel() {
        Vector<CustomFileFilter> fileFilters = new Vector<CustomFileFilter>();
        fileFilters.add(new CustomFileFilter(
                SERIALIZATION_DESCRIPTION,
                SERIALIZATION_EXTENSIONS));
        URL target = GUIUtils.showOpenFileDialog(this,
                fileFilters);
        // Check file's existance.
        if (target == null) return;
        File file = new File(target.getFile());
        if (!file.exists()) {
            printError(Messages.formatMessage(Messages
                    .getString("FILE_NOT_FOUND"), file));
            return;
        }

        setChanged(false);
        currentFile = target;

        try {
            core.loadAnalyzeModel(target);
            updateTypes();
        } catch (Exception e) {
            printError(e.getMessage());
        }
    }

    private void updateTypes() {
        ModelEditor editor = core.getAnalyzeModel()
                .getEditor();
        Collection<String> entities = editor
                .getEntityNames();
        for (String entity : entities) {
            entityTypes.add(editor.getEntityType(entity));
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see ucot.ui.UIInterface#getProgressBar()
     */
    public ProgressBarInterface getProgressBar() {
        // Initialize progress bar dialog.
        if (progressBarDialog == null) {
            progressBarDialog = new ProgressBarDialog(this,
                    PROGRESS_BAR_DIALOG_TITLE);
        }
        return progressBarDialog;
    }

    /*
     * (non-Javadoc)
     * 
     * @see ucot.ui.UIInterface#printError(java.lang.String)
     */
    public void printError(String errorMessage) {
        GUIUtils.printError(this, errorMessage);
    }

    /*
     * (non-Javadoc)
     * 
     * @see ucot.ui.UIInterface#printWarning(java.lang.String,
     *      java.lang.String)
     */
    public void printWarning(String warningMessage,
            String warningTitle) {
        GUIUtils.printWarning(this, warningMessage,
                warningTitle);
    }

    /*
     * (non-Javadoc)
     * 
     * @see ucot.ui.UIInterface#printWarning(java.lang.String)
     */
    public void printWarning(String warningMessage) {
        GUIUtils.printWarning(this, warningMessage);
    }

    /*
     * (non-Javadoc)
     * 
     * @see ucot.ui.UIInterface#printError(java.lang.String,
     *      java.lang.String)
     */
    public void printError(String errorMessage,
            String errorTitle) {
        GUIUtils.printError(this, errorMessage, errorTitle);

    }

    /**
     * <p>
     * Sets the GUI's menus and other vital elements
     * disabled. This is currently necessary because
     * otherwice the user could screw up the whole model or
     * the program by performing unexpected actions while
     * analyzing for previous use cases or something
     * similiar.
     * </p>
     * 
     * @param disabled
     *            True if GUI should be disabled, false if
     *            GUI should be enabled.
     */
    public void setDisabled(boolean disabled) {
        this.analyzeModelTree.setEnabled(!disabled);
        this.useCaseTree.setEnabled(!disabled);
        this.menu.setEnabled(!disabled);
    }

    /**
     * <p>
     * Method for creating the URL from the properties file,
     * which is the same as the class name with an .XML
     * extension.
     * </p>
     * 
     * @return URL to the properties file.
     */
    private URL getPropertiesURL() {
        String filename = this.getClass().toString();
        filename = filename.substring(filename
                .lastIndexOf('.') + 1)
                + ".xml";
        File propertiesFile = new File(filename);
        try {
            propertiesFile.createNewFile();
        } catch (IOException e) {
        }

        try {
            return propertiesFile.toURL();
        } catch (MalformedURLException e) {
            return null;
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see ucot.ModulePropertyInterface#getProperties()
     */
    public synchronized Properties getProperties() {
        return properties;
    }

    /*
     * (non-Javadoc)
     * 
     * @see ucot.ModulePropertyInterface#setProperties(java.util.Properties)
     */
    public synchronized void setProperties(
            Properties properties) {
        this.properties = PropertiesTools.merge(properties,
                this.properties);
    }

    /*
     * (non-Javadoc)
     * 
     * @see ucot.ModulePropertyInterface#applyProperties()
     */
    public synchronized void applyProperties()
            throws BadPropertyValueException {
        // TODO Should this do something else?
        dotPanel.colorTheme.changeColor(properties
                .getProperty(PROPERTY_DOT_COLOR));
        dotPanel.colorTheme.changeHighlight(properties
                .getProperty(PROPERTY_DOT_HIGHLIGHT_COLOR));
        dotPanel.updateModel(core.getAnalyzeModel());
    }

    /**
     * Updates the properties base don the internal state of
     * this object.
     */
    private void updateProperties() {
        // update types also to properties
        StringBuffer propertyStringBuf = new StringBuffer();
        for (String type : entityTypes) {
            propertyStringBuf.append(type);
            propertyStringBuf.append(", ");
        }
        // remove last 2 chars
        String propertyString = propertyStringBuf
                .substring(0, Math.max(propertyStringBuf
                        .length() - 2, 0));

        // update types into properties..
        properties.setProperty(
                GraphicalUI.PROPERTY_ENTITY_TYPES,
                propertyString);
    }

    /*
     * (non-Javadoc)
     * 
     * @see ucot.ModulePropertyInterface#saveProperties()
     */
    public synchronized void saveProperties()
            throws IOException {
        updateProperties();

        PropertiesTools.saveProperties(properties,
                this.propertiesURL);
    }

    /*
     * (non-Javadoc)
     * 
     * @see ucot.ModulePropertyInterface#loadProperties()
     */
    public synchronized void loadProperties()
            throws IOException {
        properties = loadDefaultProperties();
        properties = PropertiesTools.merge(PropertiesTools
                .loadProperties(loadDefaultProperties()
                        .propertyNames(),
                        this.propertiesURL), properties);
    }

    /*
     * (non-Javadoc)
     * 
     * @see ucot.ModulePropertyInterface#loadDefaultProperties()
     */
    public Properties loadDefaultProperties() {
        Properties result = new Properties();
        result.setProperty(PROPERTY_DOT_PATH,
                "./dot/dot.exe");
        result.setProperty(PROPERTY_EPS_TO_PDF_PATH,
                "./dot/epstopdf.exe");
        result.setProperty(PROPERTY_TEMP_INPUT_FILE,
                "temp.dot");
        result.setProperty(PROPERTY_TEMP_OUTPUT_FILE,
                "graph.png");
        result.setProperty(PROPERTY_TEMP_EPS_FILE,
                "temp.eps");
        result.setProperty(PROPERTY_DOT_COLOR,
                "BLACK_ON_WHITE");
        result.setProperty(PROPERTY_DOT_HIGHLIGHT_COLOR,
                "GREEN");
        result.setProperty(PROPERTY_ENTITY_TYPES, "");
        result.setProperty(PROPERTY_GRAPH_FONT_SIZE, "10");
        // TODO Set (other) default properties here.
        return result;
    }

    /**
     * <p>
     * Returns the <code>ControlInterface</code> this UI
     * uses to control the program.
     * </p>
     * 
     * @return The control interface.
     */
    public ControlInterface getControlInterface() {
        return core;
    }

    /**
     * <p>
     * Returns dot panel's color theme.
     * </p>
     * 
     * @return Dot panel's color theme
     */
    public DotColorTheme getColorTheme() {
        return dotPanel.colorTheme;
    }

}
