/*
 *  Copyright (C) 2006  UCOT
 *
 *  Distributed under the terms of the MIT License.
 *  See "LICENCE" or http://www.opensource.org/licenses/mit-license.php for details.
 */
package ucot.ui.gui.dialog;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.Vector;
import java.util.logging.Logger;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;

import ucot.Messages;
import ucot.model.AnalyzeModel;
import ucot.model.ModelEditor;
import ucot.model.NoSuchEntityException;
import ucot.ui.gui.GraphicalUI;
import ucot.ui.gui.dialog.entitytab.AttributesPanel;
import ucot.ui.gui.dialog.entitytab.ChildrenPanel;
import ucot.ui.gui.dialog.entitytab.EntityPropertiesEditor;
import ucot.ui.gui.dialog.entitytab.MethodsPanel;
import ucot.ui.gui.dialog.entitytab.ParentsPanel;

/**
 * <p>
 * Dialog for modifying a single entity's properties in the
 * analyze model. User can modify entity's name, its
 * methods, attributes and parents with this dialog and when
 * modifications are done, the modified analyze model is
 * returned.
 * </p>
 * 
 * <pre>
 *   Usage:
 *     - first initialize an EntityPropertiesDialog objecs as any other object. 
 *     - then call method:
 *         modifyEntityProperties(Entity to edit, AnalyzeModel)
 *       which returns the modified analyze model.
 * </pre>
 * 
 * @author ilanliuk, tujupien, pajumasu.
 */
public class EntityPropertiesDialog extends JDialog {

    private static final long serialVersionUID = -3386954136246879640L;

    private GraphicalUI owner;

    private String selectedEntityName;

    private JTabbedPane entityPropertiesTabbedPane;

    private AnalyzeModel analyzeModel;

    /** The entity name text field. */
    private JTextField entityName = new JTextField(
            // Default entity name for new entities.
            Messages
                    .getString("EntityPropertiesDialog.ENTITY_NAME"));

    /** The entity type combobox. */
    private JComboBox entityType = new JComboBox();

    private final static String ENTITY_NAME = Messages
            .getString("EntityPropertiesDialog.ENTITY_NAME");

    private final static String ENTITY_TYPE = Messages
            .getString("EntityPropertiesDialog.ENTITY_TYPE");

    private final static String ENTITY_NO_TYPE_DESCRIPTION = Messages
            .getString("EntityPropertiesDialog.ENTITY_NO_TYPE_DESCRIPTION");

    /** The header for this dialog. */
    protected final static String DIALOG_HEADER = Messages
            .getString("EntityPropertiesDialog.DIALOG_HEADER");

    /** Log message for informing entitys name change. */
    protected final static String ENTITY_NAME_CHANGED_LOG_MESSAGE = Messages
            .getString("EntityPropertiesDialog.ENTITY_NAME_CHANGED_LOG_MESSAGE");

    /** Text for the ok button. */
    protected static String MAIN_OK_BUTTON = Messages
            .getString("EntityPropertiesDialog.MAIN_OK_BUTTON");

    /** Text for the cancel button. */
    protected static String MAIN_CANCEL_BUTTON = Messages
            .getString("EntityPropertiesDialog.MAIN_CANCEL_BUTTON");

    /** Text for the delete entity button. */
    protected static String DELETE_ENTITY_BUTTON = Messages
            .getString("EntityPropertiesDialog.DELETE_ENTITY_BUTTON");

    /** Header for the main properties. */
    protected final static String MAIN_PROPERTIES_HEADER = Messages
            .getString("EntityPropertiesDialog.MAIN_PROPERTIES_HEADER");

    public final static String METHODS_TAB_KEY = "METHODS";

    public final static String PARENTS_TAB_KEY = "PARENTS";

    public final static String CHILDREN_TAB_KEY = "CHILDREN";

    public final static String ATTRIBUTES_TAB_KEY = "ATTRIBUTES";

    // Button action identifiers.
    protected final static String MAIN_OK_BUTTON_ACTION = "MAIN_OK";

    protected final static String MAIN_CANCEL_BUTTON_ACTION = "MAIN_CANCEL";

    protected final static String DELETE_ENTITY_BUTTON_ACTION = "DELETE_ENTITY";

    private Object[] tabs = { new MethodsPanel(),
            new ParentsPanel(), new ChildrenPanel(),
            new AttributesPanel() };

    private String[] tabKeys = { METHODS_TAB_KEY,
            PARENTS_TAB_KEY, CHILDREN_TAB_KEY,
            ATTRIBUTES_TAB_KEY };

    private Map<String, JComponent> tabMap = new HashMap<String, JComponent>();

    private final static Logger logger = Logger
            .getLogger("ucot.gui");

    private Collection<EntityPropertiesEditor> entityTabs = new Vector<EntityPropertiesEditor>();

    /**
     * ActionListener for all button events within the
     * dialog.
     */
    private ActionListener buttonListener = new ActionListener() {
        private ModelEditor editor;

        /**
         * This method updates the entity name if it has
         * changed.
         */
        private void updateEntityName() {
            // Uppcase the first letter.
            String newName = entityName.getText().trim();
            char firstLetter = Character
                    .toUpperCase(newName.charAt(0));
            newName = firstLetter + newName.substring(1);

            if (!selectedEntityName.equals(newName)
                    || !analyzeModel.getEditor()
                            .containsEntity(newName)) {

                logger.fine(Messages.formatMessage(
                        ENTITY_NAME_CHANGED_LOG_MESSAGE,
                        selectedEntityName, newName));

                owner.setChanged(true);

                if (analyzeModel.getEditor()
                        .containsEntity(newName)) {
                    // There is already entity by this name.
                    // So we do a merge.
                    // We add information about the entity
                    // we
                    // are going to merge with to te tables
                    // so the information is not deleted.
                    // Then we just remove the old entity.

                    for (EntityPropertiesEditor tab : entityTabs) {
                        tab.load(analyzeModel, newName);
                    }
                    editor.removeEntity(selectedEntityName);
                } else {
                    // Now either add a new entity to the
                    // analyze model (new
                    // entity) or
                    // change the name of an already
                    // existing entity (modify
                    // entity).
                    if (!analyzeModel.getEditor()
                            .containsEntity(
                                    selectedEntityName)) {
                        editor.addEntity(newName);
                    } else {
                        try {
                            editor.changeEntityName(
                                    selectedEntityName,
                                    newName);
                        } catch (NoSuchEntityException e) {
                            e.printStackTrace();
                        }
                    }
                }

                // The corresponding class variable must be
                // updated if
                // entity name was changed.
                selectedEntityName = newName;
            }
        }

        /**
         * This method updates the entity name if it has
         * changed.
         */
        private void updateEntityType() {

            Object selected = entityType.getSelectedItem();
            if (selected == null) return;

            String newType = selected.toString();
            if ("".equals(newType)) return;

            if (ENTITY_NO_TYPE_DESCRIPTION.equals(newType)) {
                analyzeModel.getEditor().setEntityType(
                        selectedEntityName, null);
                return;
            }

            analyzeModel.getEditor().setEntityType(
                    selectedEntityName, newType);
        }

        /**
         * Action performed: User clicked the main OK
         * button.
         */
        private void mainOKButtonClicked() {
            if (entityName.getText().length() <= 0) {
                owner
                        .printWarning("Each entity must have a name!");
                return;
            }

            analyzeModel.getEditor()
                    .updationStartedSignal();

            // Update entity name.
            updateEntityName();

            // Update entity type.
            updateEntityType();

            // Update changed methods to the analyze model.
            for (EntityPropertiesEditor tab : entityTabs) {
                tab.save(analyzeModel, selectedEntityName);
            }
            analyzeModel.getEditor().readySignal();

            // Vanish dialog.
            setVisible(false);
        }

        /**
         * Action performed: User clicked the main cancel
         * button.
         */
        private void mainCancelButtonClicked() {
            // This seems to be enough to close modal
            // dialog,
            // don't know for sure (ilari)

            // Checked here if new entity has already
            // been created that it is removed..

            // Vanish dialog.
            setVisible(false);
        }

        /**
         * Action performed: User clicked the 'delete
         * entity' button.
         */
        private void deleteEntityButtonClicked() {
            // Delete the selected entity and all relations
            // from and to it.

            analyzeModel.getEditor().removeEntity(
                    selectedEntityName);

            // Vanish dialog.
            setVisible(false);
        }

        /**
         * This method is called when an action is performed
         * and it calls the corresponding methods to handle
         * the action.
         */
        public void actionPerformed(ActionEvent e) {
            editor = analyzeModel.getEditor();

            if (MAIN_OK_BUTTON_ACTION.equals(e
                    .getActionCommand())) {
                mainOKButtonClicked();
                return;
            }

            if (MAIN_CANCEL_BUTTON_ACTION.equals(e
                    .getActionCommand())) {
                mainCancelButtonClicked();
                return;
            }

            if (DELETE_ENTITY_BUTTON_ACTION.equals(e
                    .getActionCommand())) {
                deleteEntityButtonClicked();
                return;
            }

        }

    };

    /**
     * <p>
     * <code>EntityPropertiesDialog</code> constructor.
     * This creates the whole dialog, but the actual
     * contents of the all fields will be set later in the
     * initialization method.
     * <p>
     * 
     * @param owner
     *            This dialog's owner component.
     * @throws HeadlessException
     *             Exception is thrown if the superclass
     *             initialization goes wrong.
     */
    public EntityPropertiesDialog(GraphicalUI owner)
            throws HeadlessException {
        super(owner, DIALOG_HEADER);
        this.owner = owner;

        // this.setPreferredSize(new Dimension(500, 500));
        this.setLayout(new BorderLayout());

        // Create all required panels and set layouts for
        // them.
        entityPropertiesTabbedPane = new JTabbedPane();
        JPanel mainButtonsPanel = new JPanel();
        JPanel mainPropertiesPanel = new JPanel();
        mainPropertiesPanel.setLayout(new BorderLayout());

        int count = 0;
        for (Object tab : tabs) {
            if (tab instanceof EntityPropertiesEditor) {
                entityTabs
                        .add((EntityPropertiesEditor) tab);
                tabMap.put(tabKeys[count],
                        ((EntityPropertiesEditor) tab)
                                .getComponent());
            } else {
                logger
                        .severe(Messages
                                .formatMessage(
                                        Messages
                                                .getString("EntityPropertiesDialog.UNKNONW_TAB_TYPE"),
                                        tab.getClass()
                                                .toString()));
            }
            count++;
        }

        // Divide the dialog into four sections. Top section
        // for entity
        // properties,
        // left (center) section for entity's methods table
        // and bottom section
        // for
        // general dialog buttons.
        this.getContentPane().add(mainButtonsPanel,
                BorderLayout.SOUTH);
        this.getContentPane().add(mainPropertiesPanel,
                BorderLayout.NORTH);
        this.getContentPane().add(
                entityPropertiesTabbedPane,
                BorderLayout.CENTER);
        for (EntityPropertiesEditor tab : entityTabs) {
            entityPropertiesTabbedPane.addTab(tab
                    .getTabName(), tab.getComponent());
        }

        // Initialize entity properties' container.
        mainPropertiesPanel
                .setBorder(BorderFactory
                        .createTitledBorder(MAIN_PROPERTIES_HEADER));

        JPanel mainPropertiesLabelsPanel = new JPanel();
        JPanel mainPropertiesTextsPanel = new JPanel();

        GridBagLayout mainPropertiesLayout = new GridBagLayout();
        mainPropertiesPanel.setLayout(mainPropertiesLayout);

        GridBagConstraints c = new GridBagConstraints();

        c.fill = GridBagConstraints.EAST;
        c.gridx = 0;
        c.gridy = 0;
        c.gridwidth = 1;
        c.gridheight = 1;
        c.weightx = 0;
        c.ipadx = 5;
        mainPropertiesPanel.add(mainPropertiesLabelsPanel,
                c);

        c = new GridBagConstraints();
        c.gridx = GridBagConstraints.RELATIVE;
        c.gridy = 0;
        c.gridwidth = 1;
        c.gridheight = 1;
        c.weightx = 1;
        c.fill = GridBagConstraints.HORIZONTAL;
        mainPropertiesPanel
                .add(mainPropertiesTextsPanel, c);

        mainPropertiesTextsPanel
                .setLayout(new BorderLayout());
        mainPropertiesTextsPanel.add(entityName,
                BorderLayout.NORTH);
        mainPropertiesTextsPanel.add(entityType,
                BorderLayout.SOUTH);

        mainPropertiesLabelsPanel
                .setLayout(new BorderLayout());
        JLabel entityNameLabel = new JLabel(ENTITY_NAME);
        mainPropertiesLabelsPanel.add(entityNameLabel,
                BorderLayout.NORTH);
        JLabel entityTypeLabel = new JLabel(ENTITY_TYPE);
        mainPropertiesLabelsPanel.add(entityTypeLabel,
                BorderLayout.SOUTH);

        JButton b = new JButton(MAIN_OK_BUTTON);
        b.setActionCommand(MAIN_OK_BUTTON_ACTION);
        b.addActionListener(buttonListener);
        mainButtonsPanel.add(b);
        b = new JButton(MAIN_CANCEL_BUTTON);
        b.setActionCommand(MAIN_CANCEL_BUTTON_ACTION);
        b.addActionListener(buttonListener);
        mainButtonsPanel.add(b);
        b = new JButton(DELETE_ENTITY_BUTTON);
        b.setActionCommand(DELETE_ENTITY_BUTTON_ACTION);
        b.addActionListener(buttonListener);
        mainButtonsPanel.add(b);
        // <-- Button creation section ends here.

        this.pack();
    }

    /**
     * <p>
     * Method to spawn the
     * <code>EntityPropertiesDialog</code> and stay modal
     * until user closes it. Modified analyze model is
     * returned.
     * </p>
     * 
     * @param entityName
     *            name of the entity to edit.
     * @param model
     *            <code>AnalyzeModel</code> to edit.
     * @return Modified analyze model as an
     *         <code>AnalyzeModel</code> object.
     */
    public AnalyzeModel modifyEntityProperties(
            String entityName, AnalyzeModel model) {

        this.selectedEntityName = entityName;
        this.analyzeModel = model;

        initializeDialog();

        // This shows the modal dialog, and stops here until
        // the dialog is closed.
        this.setVisible(true);

        // The analyzemodel is modified when OK button is
        // clicked
        // in the buttonListener.

        // Return the modified model.
        return this.analyzeModel;
    }

    /**
     * This method initializes the dialog and all its
     * dynamic components.
     */
    private void initializeDialog() {

        // Set focus to the first tab.
        entityPropertiesTabbedPane.setSelectedIndex(0);

        // Add available entities to entitiesArray which is
        // used for generating
        // the ComboBox editor for some columns. Also set
        // the cardinality
        // editors
        // to cardinality columns.
        Set<String> entities = analyzeModel.getEditor()
                .getEntityNames();
        String[] entitiesArray = new String[entities.size()];
        entitiesArray = entities.toArray(entitiesArray);

        entityName.setText(selectedEntityName);

        //
        // Initialize EntityType-ComboBox
        // clear old ones from combobox
        entityType.removeAllItems();
        // add empty line so user can select empty typename
        entityType.addItem(ENTITY_NO_TYPE_DESCRIPTION);
        for (String type : owner.entityTypes) {
            entityType.addItem(type);
        }
        // select current entity type from list
        if ("".equals(analyzeModel.getEditor()
                .getEntityType(selectedEntityName))) {
            entityType.setSelectedIndex(0);
        } else {
            entityType.setSelectedItem(analyzeModel
                    .getEditor().getEntityType(
                            selectedEntityName));
        }

        // Load table data from analyze model.

        for (EntityPropertiesEditor tab : entityTabs) {
            tab.clear();
            tab.load(analyzeModel, selectedEntityName);
        }
    }

    /**
     * <p>
     * Constructs, initializes and spawns an entity
     * properties dialog for the given entity in the given
     * analyze model. The given model is updated with the
     * user's modifications and then returned.
     * </p>
     * 
     * @param owner
     *            Owner component of the to generated entity
     *            properties dialog.
     * @param entityName
     *            The name of the entity to be edited.
     * @param model
     *            The analyze model which is being modified.
     * @return Modified analyze model as an
     *         <code>AnalyzeModel</code> object.
     */
    public static AnalyzeModel showDialog(
            GraphicalUI owner, String entityName,
            AnalyzeModel model) {
        EntityPropertiesDialog entityPropertiesDialog = new EntityPropertiesDialog(
                owner);
        return entityPropertiesDialog
                .modifyEntityProperties(entityName, model);
    }

    /**
     * Shows the current tab.
     * 
     * @param key
     *            The key for the current tab.
     */
    public void showTabFor(String key) {
        // Default is the first tab.
        Component comp = null;

        // If we have key check for the component.
        if (null != key) {
            comp = tabMap.get(key);
        }

        // Default is the first tab. Use it if nothin else
        // can be used.
        if (null == comp) comp = entityPropertiesTabbedPane
                .getComponentAt(0);

        // Set the tab.
        entityPropertiesTabbedPane
                .setSelectedComponent(comp);
    }
}
