/*
 *  Copyright (C) 2006  UCOT
 *
 *  Distributed under the terms of the MIT License.
 *  See "LICENCE" or http://www.opensource.org/licenses/mit-license.php for details.
 */
package ucot.ui.gui.dialog.entitytab;

import java.util.HashSet;
import java.util.Set;
import java.util.Vector;
import java.util.logging.Logger;

import ucot.Messages;
import ucot.model.AnalyzeModel;
import ucot.model.AnalyzeModelException;
import ucot.model.ModelEditor;
import ucot.model.NoSuchEntityException;

/**
 * <p>
 * This panel allows user to edit entitys parents.
 * </p>
 */
public class ParentsPanel extends JTableEntityPropertiesTab {

    private static final long serialVersionUID = 4325462850141268992L;

    /** Log message for informing creation of new parents. */
    protected final static String NEW_PARENTS_LOG_MESSAGE = Messages
            .getString("EntityPropertiesDialog.NEW_PARENTS_LOG_MESSAGE");

    /**
     * Log message for informing deletion of removed
     * parents.
     */
    protected final static String DELETED_PARENTS_LOG_MESSAGE = Messages
            .getString("EntityPropertiesDialog.DELETED_PARENTS_LOG_MESSAGE");

    /** Text for new parent button. */
    protected static String NEW_PARENT_BUTTON = Messages
            .getString("EntityPropertiesDialog.NEW_PARENT_BUTTON");

    /** Text for delete parent button. */
    protected static String DELETE_PARENT_BUTTON = Messages
            .getString("EntityPropertiesDialog.DELETE_PARENT_BUTTON");

    protected final static String NEW_PARENT_BUTTON_ACTION = "NEW_PARENT";

    protected final static String DELETE_PARENT_BUTTON_ACTION = "DELETE_PARENT";

    /** Headers for the method table. */
    protected final static String[] PARENTS_TABLE_COLUMNS = {
    // Header for method name column.
    Messages
            .getString("EntityPropertiesDialog.PARENT_COLUMN_NAME") };

    /** Header for the parents tab. */
    protected static String PARENTS_TAB_HEADER = Messages
            .getString("EntityPropertiesDialog.PARENTS_TAB_HEADER");

    private final static Logger logger = Logger
            .getLogger("ucot.gui");

    /**
     * <p>
     * Creates the panel using the localized strings readed
     * from the <code>Messages</code> object.
     * </p>
     */
    public ParentsPanel() {
        super(PARENTS_TAB_HEADER, PARENTS_TABLE_COLUMNS);
        // Create required buttons. -->
        addButton(NEW_PARENT_BUTTON,
                NEW_PARENT_BUTTON_ACTION);
        addButton(DELETE_PARENT_BUTTON,
                DELETE_PARENT_BUTTON_ACTION);
    }

    /**
     * Action performed: User clicked the 'new parent'
     * button.
     */
    private void newParentButtonClicked() {
        // Add new parent to the parents table.
        Vector<String> row = new Vector<String>();
        row.add("");
        getModel().addRow(row);
    }

    /**
     * Action performed: User clicked the 'delete parent'
     * button.
     */
    private void deleteParentButtonClicked() {
        // Stop cell editing.
        if (getTable().isEditing()) getTable()
                .getCellEditor().stopCellEditing();
        // Delete selected row from the parents table.
        if (getTable().getSelectedRow() < 0) return;
        getModel().removeRow(getTable().getSelectedRow());
    }

    /**
     * This method is called when an action is performed and
     * it calls the corresponding methods to handle the
     * action.
     */
    public void action(String cmd) {
        if (NEW_PARENT_BUTTON_ACTION.equals(cmd)) {
            newParentButtonClicked();
            return;
        }

        if (DELETE_PARENT_BUTTON_ACTION.equals(cmd)) {
            deleteParentButtonClicked();
            return;
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see ucot.ui.gui.dialog.entitytab.EntityPropertiesEditor#save(ucot.model.AnalyzeModel,
     *      java.lang.String)
     */
    public void save(AnalyzeModel analyzeModel,
            String saveEntityName) {

        super.save(analyzeModel, saveEntityName);

        ModelEditor editor = analyzeModel.getEditor();
        Set<String> oldParents = editor
                .getParents(saveEntityName);
        Set<String> currentParents = new HashSet<String>();

        // Initialize current parents.
        for (int i = 0; i < tableModel.getRowCount(); i++) {
            String parentName = ((String) tableModel
                    .getValueAt(i, 0)).trim();
            // Paretn cannot be zero length or the object
            // itself cant be its
            // parent.
            if (parentName.length() > 0
                    && !parentName.equals(saveEntityName)) {
                currentParents.add(parentName);
            }
        }

        // Calculate new parents and deleted parents.
        Set<String> newParents = new HashSet<String>();
        newParents.addAll(currentParents);
        newParents.removeAll(oldParents);
        Set<String> deletedParents = new HashSet<String>();
        deletedParents.addAll(oldParents);
        deletedParents.removeAll(currentParents);
        if (newParents.size() > 0
                || deletedParents.size() > 0) {
            // owner.setChanged(true);
        }

        logger.fine(Messages.formatMessage(
                NEW_PARENTS_LOG_MESSAGE, newParents));
        logger.fine(Messages
                .formatMessage(DELETED_PARENTS_LOG_MESSAGE,
                        deletedParents));

        Set<String> entityNames = editor.getEntityNames();
        // Add new parents to the analyze model.
        for (String parentName : newParents) {
            if (!parentName.equals(saveEntityName)) {
                // Dont try to add non exitent parent.
                if (entityNames.contains(parentName)) {
                    try {
                        editor.addParent(saveEntityName,
                                parentName);
                    } catch (NoSuchEntityException e) {
                        e.printStackTrace();
                    }
                }
            }
        }

        // Remove non existing parent relationships from the
        // analyze model.
        for (String parentName : deletedParents) {
            try {
                editor.removeParent(saveEntityName,
                        parentName);
            } catch (AnalyzeModelException e) {
                e.printStackTrace();
            }
        }

    }

    /*
     * (non-Javadoc)
     * 
     * @see ucot.ui.gui.dialog.entitytab.EntityPropertiesEditor#load(ucot.model.AnalyzeModel,
     *      java.lang.String)
     */
    public void load(AnalyzeModel analyzeModel,
            String loadEntityName) {
        super.load(analyzeModel, loadEntityName);
        updateCellEditor(analyzeModel, loadEntityName, 0,
                false);
        Set<String> parents = analyzeModel.getEditor()
                .getParents(loadEntityName);
        for (String parent : parents) {
            if (parent.equals(loadEntityName)) parent = SELF_POINTER_NAME;
            Vector<String> row = new Vector<String>();
            row.addElement(parent);
            getModel().addRow(row);
        }
    }

}
