/*
 *  Copyright (C) 2006  UCOT
 *
 *  Distributed under the terms of the MIT License.
 *  See "LICENCE" or http://www.opensource.org/licenses/mit-license.php for details.
 */

package ucot.ui.gui.dot;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Collections;
import java.util.HashSet;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;
import java.util.Set;
import java.util.Stack;
import java.util.Vector;
import java.util.logging.Logger;

import javax.imageio.ImageIO;
import javax.swing.JLabel;
import javax.swing.JPanel;

import ucot.Messages;
import ucot.model.AnalyzeModel;
import ucot.model.AnalyzeModelException;
import ucot.model.ModelEditor;
import ucot.model.Updation;
import ucot.model.Updation.UpdationType;
import ucot.model.object.ObjectAnalyzeModel;
import ucot.ui.gui.GraphicalUI;
import ucot.utils.FileTools;
import ucot.utils.StringTools;

/**
 * <p>
 * A dot panel class inherited from <code>JPanel</code>
 * class which is used for drawing dot graphs from the given
 * analyze model.
 * </p>
 * <p>
 * This class implements the <code>Observer</code>
 * interface which allows this panel to keep track of the
 * current status of analyze model. Panel automatically
 * updates the graph when the model changes
 * </p>
 * 
 * <p>
 * <code>DotPanel</code> contains also a method
 * <code>highlight</code> for highlighting any submodels
 * from the given <code>Analyzemodel</code>.
 * </p>
 * 
 * @see AnalyzeModel
 * 
 * @author ilanliuk, tujupien, vevijopi.
 */
public class DotPanel extends JPanel implements Observer {

    /**
     * This class is a container for a dot job which
     * includes all required information for rendering both
     * highlighted and regular dot graphs. A container like
     * this is required when dot execution is done in
     * threads.
     */
    private class DotJob {
        private AnalyzeModel analyzeModel = new ObjectAnalyzeModel();

        private AnalyzeModel highlightModel = new ObjectAnalyzeModel();

        private Set<String> entitiesToBeHighlighted = new HashSet<String>();

        private Set<String> entitiesInCurrentModel =

        new HashSet<String>();

        private Set<String> entitiesRequestedForHighlighting = new HashSet<String>();

        private Set<Vector<String>> parentsToBeHighlighted = new HashSet<Vector<String>>();

        private Set<Vector<String>> parentsInCurrentModel = new HashSet<Vector<String>>();

        private Set<Vector<String>> parentsRequestedForHighlighting = new HashSet<Vector<String>>();

        private Set<Vector<String>> methodsToBeHighlighted = new HashSet<Vector<String>>();

        private Set<Vector<String>> methodsInCurrentModel = new HashSet<Vector<String>>();

        private Set<Vector<String>> methodsRequestedForHighlighting = new HashSet<Vector<String>>();

        private Set<Vector<String>> attributesToBeHighlighted = new HashSet<Vector<String>>();

        private Set<Vector<String>> attributesInCurrentModel = new HashSet<Vector<String>>();

        private Set<Vector<String>> attributesRequestedForHighlighting = new HashSet<Vector<String>>();
    }

    /**
     * <p>
     * Class for giving needed info for
     * <code>executeExport()</code> method.
     * </p>
     * <p>
     * Needs to stored in own class cause
     * <code>ExportJob</code>s are stored in stack before
     * <code>executeExport()</code> accesses them.
     * </p>
     * 
     * @author tujupien
     * 
     */
    private class ExportJob {
        private BufferedImage graph = new BufferedImage(1,
                1, BufferedImage.TYPE_INT_RGB);

        private ExportImageType imageType = DEFAULT_EXPORT_IMAGE_TYPE;

        private File targetFile = new File(
                owner
                        .getProperties()
                        .getProperty(
                                GraphicalUI.PROPERTY_TEMP_OUTPUT_FILE)
                        + ".tmp");

    }

    public static final long serialVersionUID = 3899637467435823526L;

    private static boolean THREAD_RUNNING = true;

    private static Stack<DotJob> DOT_JOB_STACK = new Stack<DotJob>();

    private static Thread EXECUTING_DOT_THREAD = null;

    private static Stack<ExportJob> EXPORT_JOB_STACK = new Stack<ExportJob>();

    private static Thread EXECUTING_EXPORT_THREAD = null;

    private Logger logger = Logger
            .getLogger("ucot.ui.gui.dot");

    /**
     * Imageformats that are usable when exporting graph as
     * image.
     * 
     * @author ilanliuk
     * 
     */
    public enum ExportImageType {
        PNG, JPG, EPS, PDF
    };

    public static final ExportImageType DEFAULT_EXPORT_IMAGE_TYPE = ExportImageType.PNG;

    protected long timeElapsed;

    protected JLabel timerComponent;

    protected int fontSize = 5;

    public DotColorTheme colorTheme = new DotColorTheme();

    public final static int SCROLL_SPEED = 20;

    private BufferedImage modelImage = null;

    protected GraphicalUI owner;

    private boolean horizontalLayout = true;

    private boolean isUpdating;

    /**
     * <p>
     * Constructor for <code>DotPanel</code>. This
     * constructor adds an observer to the analyze model and
     * sets its own status to updating.
     * </p>
     * 
     * TODO fix this param, it was analyzeModel before
     * 
     * @param owner
     *            <code>AnalyzeModel</code> which this
     *            <code>DotPanel</code> should draw.
     */

    public DotPanel(GraphicalUI owner) {
        this.owner = owner;
        owner.getControlInterface().getAnalyzeModel()
                .addObserver(this);
        this.isUpdating = true;
    }

    /**
     * <p>
     * Sets wether or not the dot should use the horizontal
     * layout for the entities and their relationships.
     * </p>
     * 
     * @param horizontalLayout
     *            New value for horizontal layout, true
     *            means yay for horizontal layouting.
     */
    public void setHorizontalLayout(boolean horizontalLayout) {
        this.horizontalLayout = horizontalLayout;
        highlight(null);
    }

    /**
     * <p>
     * Returns the current value of horizontal layouting.
     * </p>
     * 
     * @return <code>Boolean</code> <code>true</code> if
     *         horizontal layout is in use, else
     *         <code>false</code>.
     */
    public boolean getHorizontalLayout() {
        return this.horizontalLayout;
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.util.Observer#update(java.util.Observable,
     *      java.lang.Object)
     */
    public void update(Observable observableObject,
            Object updationArg) {
        if (!(updationArg instanceof Updation)) return;

        // Change updation status.
        Updation updation = (Updation) updationArg;
        if (updation.getType() == UpdationType.MODIFICATION_STARTED) {
            this.isUpdating = false;
        }
        if (updation.getType() == UpdationType.READY
                || updation.getType() == UpdationType.CLEAR) {
            this.isUpdating = true;
        }

        // Update graph if required.
        if (this.isUpdating) {
            this.updateModel(owner.getControlInterface()
                    .getAnalyzeModel());
        }
    }

    /**
     * <p>
     * This method writes the graph given as a buffered
     * image to the given file in the format specified by
     * imageType.
     * </p>
     * 
     * @param graph
     *            Graph as <code>BufferedImage</code> to
     *            be written.
     * @param target
     *            <code>File</code> where the graph should
     *            be saved.
     * @param imageType
     *            File format to use when saving as
     *            <code>ExportImageType</code>.
     * @throws IOException
     *             If something goes wrong with the writing.
     */
    private void saveImage(BufferedImage graph,
            File target, ExportImageType imageType)
            throws IOException {
        ImageIO.write(graph, imageType.toString(), target);
    }

    /**
     * <p>
     * This method generates a new graph from the analyze
     * model and exports it to the given target file in a
     * format specified by imageType.
     * </p>
     * 
     * @param target
     *            <code>File</code> where the graph should
     *            be saved.
     * @param imageType
     *            File format to use when saving as
     *            <code>ExportImageType</code>.
     * @throws IOException
     *             If something goes wrong with the writing.
     */
    private void saveExport(File target,
            ExportImageType imageType) throws IOException {
        DotJob job = new DotJob();
        try {
            job.analyzeModel.getEditor().merge(
                    owner.getControlInterface()
                            .getAnalyzeModel());
        } catch (AnalyzeModelException e) {
            // print exceptions stack trace into log for
            // debugging
            // and print exceptions message to console.
            logger.warning(e.getMessage());
            logger.finest(StringTools
                    .exceptionStackTrace(e));
        }

        File dotFile = createDotFile(job);
        String args = "";

        /*
         * Check in what image format the graph should be
         * saved and create execute command for Dot
         * according to it.
         */
        switch (imageType) {
        case EPS:
            args = "-Gsize=8,10.7 -Gmargin=0 -Nfontsize=7 -Efontsize=6 "
                    + "-Gpagedir=TL -Gremincross=true -Gconcentrate=true -Tps2";

            runDot(dotFile, target, args);
            break;
        case PDF:
            // File epsImage = new
            // File(this.owner.getProperties().
            // getProperty(GraphicalUI.PROPERTY_TEMP_EPS_FILE));

            File epsImage = FileTools.changeExtension(
                    target, "eps");
            args = "-Gsize=8,10.7 -Gmargin=0 -Nfontsize=7 -Efontsize=6 "
                    + "-Gpagedir=TL -Gremincross=true -Gconcentrate=true -Tps2";

            runDot(dotFile, epsImage, args);
            convertEPSToPDF(epsImage, target);
            break;
        /*
         * case SVG: runDot(dotFile, target, "-Tsvg");
         * break;
         */
        case JPG:
            runDot(dotFile, target, "-Tjpg");
            break;
        default:
            break;
        }
    }

    /**
     * <p>
     * This method should be used inside a new thread. Once
     * started, this method will keep on running until the
     * <code>DotPanel</code> gets finalized and while
     * running, this method will perform all graph exporting
     * scheduled for <code>DotPanel</code>.
     * </p>
     */
    private void executeExport() {
        while (THREAD_RUNNING) {
            if (!EXPORT_JOB_STACK.isEmpty()) {

                ExportJob job = null;

                synchronized (EXPORT_JOB_STACK) {
                    job = EXPORT_JOB_STACK.pop();
                }

                if (job == null) continue;

                try {
                    // PNG and JPG exports are done in the
                    // same way. They're
                    // just being written to the disk as-is.
                    if (job.imageType == ExportImageType.PNG) {
                        saveImage(job.graph,
                                job.targetFile,
                                job.imageType);
                    }
                    // Others use a different approach and
                    // must be generated
                    // by dot instead of easily just saving
                    // the image.
                    else {
                        saveExport(job.targetFile,
                                job.imageType);
                    }
                } catch (IOException e) {
                    e.printStackTrace();
                }
            } else {
                try {
                    synchronized (EXECUTING_EXPORT_THREAD) {
                        EXECUTING_EXPORT_THREAD.wait();
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
    }

    /**
     * <p>
     * This method starts a new thread for export actions if
     * one has not been started yet. In each case a new
     * export job is pushed to the top of the export job
     * stack and the thread is notified about this action,
     * which wakes the thread and it will start doing the
     * topmost job from the stack when it wakes up.
     * </p>
     * 
     * @param target
     *            <code>File</code> where the image should
     *            be saved.
     * @param imageType
     *            Format for the image to be saved as
     *            <code>ExportImageType</code>.
     */
    public void exportImage(File target,
            ExportImageType imageType) {
        // if no usecases have been added into
        // analyzemodel modelImage is null..
        if (target == null || modelImage == null) return;

        ExportJob job = new ExportJob();
        job.graph = modelImage;
        job.imageType = imageType;
        job.targetFile = target;

        synchronized (EXPORT_JOB_STACK) {
            EXPORT_JOB_STACK.push(job);
        }
        if (EXECUTING_EXPORT_THREAD == null) {
            EXECUTING_EXPORT_THREAD = new Thread(
                    new Runnable() {
                        public void run() {
                            executeExport();
                        }
                    });
            EXECUTING_EXPORT_THREAD.start();
        } else {
            synchronized (EXECUTING_EXPORT_THREAD) {
                EXECUTING_EXPORT_THREAD.notify();
            }
        }
    }

    /**
     * <p>
     * Saves current image to given file in format that is
     * tried to guess from filename.
     * </p>
     * 
     * @param target
     *            <code>File</code> pointing the saving
     *            destination.
     */
    public void exportImage(File target) {

        // Strip extension from filename
        String filename = target.getName();
        String extension = filename.substring(filename
                .lastIndexOf(".") + 1);
        boolean imagetypeFound = false;

        // try to resolve in wich imagetype we should save
        // the image.
        for (ExportImageType imagetype : ExportImageType
                .values()) {
            if (extension.equals(imagetype.toString()
                    .toLowerCase())) {
                exportImage(target, imagetype);
                imagetypeFound = true;
                continue;
            }
        }
        if (!imagetypeFound) { throw new Error(
                "No ExportImageType found for file extension of \""
                        + target.getName() + "\"."); }
    }

    /**
     * <p>
     * Converts eps file to pdf using epstopdf program.
     * </p>
     * 
     * @param epsImage
     *            <code>File</code> pointing eps file to
     *            convert.
     * @param target
     *            <code>File</code> where the converted
     *            pdf file should be saved.
     * @throws IOException
     *             if something went wrong when accessing
     *             the files.
     */
    private void convertEPSToPDF(File epsImage, File target)
            throws IOException {
        File epstopdfExecutable = new File(
                this.owner
                        .getProperties()
                        .getProperty(
                                GraphicalUI.PROPERTY_EPS_TO_PDF_PATH));
        /*
         * String executeCommand = "\"" +
         * epstopdfExecutable.getAbsolutePath() + "\"
         * --outfile=\"" + target.getAbsolutePath() + "\"
         * \"" + epsImage.getAbsolutePath() + "\"";
         */
        // System.out.println(executeCommand);
        // Run EPS2PDF executable.
        /*
         * Miikalta terkkuja, korvaa Process
         * ProcessBuilderilla (ilmeisesti) ProcessBuilder pb =
         * new ProcessBuilder(...); Map<String, String> env =
         * pb.environment(); String path = env.get("path"); //
         * lis pathin alkuun dot-polku + pathseparator
         */

        ProcessBuilder pb = new ProcessBuilder(
                epstopdfExecutable.getName(), "--outfile="
                        + target.getName(), epsImage
                        .getName());
        Map<String, String> env = pb.environment();

        String td = new File(epstopdfExecutable
                .getCanonicalPath()).getParent();
        // insert set xxx=yyy into environment just for our
        // children
        env
                .put(
                        "PATH",
                        System.getenv("PATH")
                                + System
                                        .getProperty("path.separator")
                                + td);

        // set up the working directory, not it is not
        // called setDirectory

        pb.directory(new File(new File(target
                .getCanonicalPath()).getParent()));

        Process epsToPDFProcess = pb.start();

        // epsToPDFProcess =
        // Runtime.getRuntime().exec(executeCommand);

        if (epsToPDFProcess == null) return;

        try {
            epsToPDFProcess.waitFor();
        } catch (InterruptedException e) {
            logger.warning(e.getMessage());
            logger.finest(StringTools
                    .exceptionStackTrace(e));
            return;
        }
    }

    /**
     * <p>
     * This method runs the dot executable with the given
     * dot input file and returns the location of the image
     * file.
     * </p>
     * 
     * @param dotInputFile
     *            <code>File</code> which should be
     *            executed with Dot.
     * @param outputFile
     *            Path to the output file.
     * @param args
     *            Command line arguments for dot.
     * @return File where the image is located.
     *         <code>Null</code> is returned if something
     *         went wrog.
     */
    private File runDot(File dotInputFile, File outputFile,
            String args) {

        // FIXME: Returned File is the given outputFile.
        // No reason to return the file, would be enough
        // to return if all went well or not.

        // Ensure that we can find the dot executable.
        File dotExecutable = new File(this.owner
                .getProperties().getProperty(
                        GraphicalUI.PROPERTY_DOT_PATH));
        if (!dotExecutable.exists()) {
            logger.warning(Messages.getFormatedMessage(
                    "DotPanel.DOT_NOT_FOUND", dotExecutable
                            .getAbsolutePath()));
            return null;
        }

        // TODO Debug this on Linux.
        // Get the output from dot executable as a PNG
        // image.
        File td = new File(outputFile.getAbsoluteFile()
                .getParent());
        String executeCommand = "\""
                + dotExecutable.getAbsolutePath()
                + "\" -o \"" + outputFile.getName() + "\" "
                + args + " \""
                + dotInputFile.getAbsolutePath() + "\"";

        // add the dot execute string into log with finest
        // rating
        logger.finest(Messages.getFormatedMessage(
                "DOT_EXEC_COMMAND", executeCommand));

        // Run dot executable.
        Process dotProcess = null;
        try {
            dotProcess = Runtime.getRuntime().exec(
                    executeCommand, null, td);
            // dotProcess = pb.start();
        } catch (IOException e) {
            logger.warning(e.getMessage());
            logger.warning(StringTools
                    .exceptionStackTrace(e));
            outputFile = null;
        }

        if (dotProcess == null) return null;

        // System.out.println("1");
        try {
            dotProcess.waitFor();
        } catch (InterruptedException e) {
            logger.warning(e.getMessage());
            logger.finest(StringTools
                    .exceptionStackTrace(e));
        }
        // System.out.println("2");
        return outputFile;
    }

    /**
     * <p>
     * Overloaded <code>runDot</code> method which uses
     * the default temporary output file as a target for the
     * file.
     * </p>
     * 
     * @param dotInputFile
     *            <code>File</code> which should be
     *            executed with Dot.
     * @param args
     *            Command line arguments for dot.
     * @return <code>File</code> where the image is
     *         located. <code>Null</code> is returned if
     *         something went wrog.
     * 
     * @see #runDot(File, File, String)
     */
    private File runDot(File dotInputFile, String args) {
        File outputFile = new File(
                this.owner
                        .getProperties()
                        .getProperty(
                                GraphicalUI.PROPERTY_TEMP_OUTPUT_FILE));
        return runDot(dotInputFile, outputFile, args);
    }

    /**
     * <p>
     * Overloaded <code>runDot</code> method. This will
     * create the dot graph with default settings (PNG image
     * with the paths defined in settings XML).
     * </p>
     * 
     * @param dotInputFile
     *            <code>File</code> which should be
     *            executed with Dot.
     * @return <code>File</code> where the image is
     *         located. <code>Null</code> is returned if
     *         something went wrog.
     * 
     * @see #runDot(File, String)
     */
    private File runDot(File dotInputFile) {
        return runDot(dotInputFile, "-Tpng");
    }

    /**
     * <p>
     * Method for writing the methods of one entity to the
     * dot file.
     * <p>
     * 
     * @param job
     *            <code>DotJob</code> that contains the
     *            <code>AnalyzeModel</code> where the
     *            entity we are accessing is located.
     * @param writer
     *            <code>BufferedWriter</code> to use for
     *            writing.
     * @param entity
     *            Name of the entity which methods are being
     *            written.
     * @throws IOException
     *             If something goes wrong when accessing
     *             the file.
     */
    private void writeEntityMethods(DotJob job,
            BufferedWriter writer, String entity)
            throws IOException {

        ModelEditor editor = job.analyzeModel.getEditor();

        // get methods of this entity
        Set<String> methods = editor.getMethodNames(entity);

        try {
            for (String method : methods) {

                // get entities this method is pointing to
                Set<String> targets = editor
                        .getEntitiesInfluencedByMethod(
                                entity, method);

                for (String target : targets) {

                    // write the method relation into dot
                    // file
                    writer.write("entity"
                            + Math.abs(entity.hashCode())
                            + " -> entity"
                            + Math.abs(target.hashCode())
                            + " [label=\"" + method
                            + "\", arrowhead=\"vee\", ");
                    Vector<String> compare = new Vector<String>();

                    // check if the method relation should
                    // be hinghlighted
                    compare.add(entity);
                    compare.add(method);
                    compare.add(target);
                    writer
                            .write(colorTheme
                                    .getColorString(job.methodsToBeHighlighted != null
                                            && job.methodsToBeHighlighted
                                                    .contains(compare)));

                    // end this line
                    writer.write(", fontsize=\"" + fontSize
                            + "\"];");
                    writer.newLine();
                }
            }
        } catch (IOException e) {
            // if we caugth IOException just
            // throw it for someone else..
            throw e;
        }
    }

    /**
     * <p>
     * Method for writing the attributes of one entity to
     * the dot file. Method also highlights the attribute
     * relation if it's needed.
     * </p>
     * 
     * @param job
     *            <code>DotJob</code> that contains the
     *            <code>AnalyzeModel</code> where the
     *            entity which attributes we are accessing
     *            is located.
     * @param writer
     *            BufferedWriter to use for writing.
     * @param entity
     *            Name of the entity whose attributes are
     *            being written.
     * @throws IOException
     *             If something goes wrong when accessing
     *             the file.
     */
    private void writeEntityAttributes(DotJob job,
            BufferedWriter writer, String entity)
            throws IOException {

        ModelEditor editor = job.analyzeModel.getEditor();

        // get attributes of this entity and go through them
        Set<String> attributes = editor
                .getAttributeNames(entity);
        for (String attribute : attributes) {
            try {
                writer
                        .write("entity"
                                + Math.abs(entity
                                        .hashCode())
                                + " -> entity"
                                + Math.abs(attribute
                                        .hashCode())
                                + " [taillabel=\""
                                + editor
                                        .getAttributeFromCardinal(
                                                entity,
                                                attribute)
                                + "\", headlabel=\""
                                + editor
                                        .getAttributeToCardinal(
                                                entity,
                                                attribute)
                                + "\", arrowtail=\"diamond\", arrowsize=\"1.3\", labeldistance=\"1.7\""
                                + ", arrowhead=\"none\", ");
            } catch (AnalyzeModelException e) {
                e.printStackTrace();
            }
            Vector<String> compare = new Vector<String>();
            compare.add(entity);
            compare.add(attribute);
            writer
                    .write(colorTheme
                            .getColorString(job.attributesToBeHighlighted != null
                                    && job.attributesToBeHighlighted
                                            .contains(compare)));
            writer.write(", fontsize=\"" + fontSize
                    + "\"];");
            writer.newLine();
        }
    }

    /**
     * <p>
     * Method for writing the parents of one entity to the
     * dot file. Method also highlights the parent-child
     * relation if it's needed.
     * </p>
     * 
     * @param job
     *            <code>DotJob</code> that contains the
     *            <code>AnalyzeModel</code> where the
     *            entity which parents we are accessing is
     *            located.
     * @param writer
     *            <code>BufferedWriter</code> to use for
     *            writing.
     * @param entity
     *            Name of the entity whose parents are being
     *            written.
     * @throws IOException
     *             If something goes wrong when accessing
     *             the file.
     */
    private void writeEntityParents(DotJob job,
            BufferedWriter writer, String entity)
            throws IOException {

        // get parents of this entity.
        Set<String> parents = job.analyzeModel.getEditor()
                .getParents(entity);
        for (String parent : parents) {

            // write the parent-child relation to dot file.
            writer
                    .write("entity"
                            + Math.abs(parent.hashCode())
                            + " -> entity"
                            + Math.abs(entity.hashCode())
                            + " [arrowtail=\"onormal\", arrowhead=\"none\", arrowsize=\"1.5\", ");
            Vector<String> compare = new Vector<String>();

            // check if the relation needs to be
            // highlighted.
            compare.add(entity);
            compare.add(parent);
            writer
                    .write(colorTheme
                            .getColorString(job.parentsToBeHighlighted != null
                                    && job.parentsToBeHighlighted
                                            .contains(compare)));

            // end this line
            writer.write(", fontsize=\"" + fontSize
                    + "\"];");
            writer.newLine();
        }
    }

    /**
     * <p>
     * Method for writing all entities to the dot file with
     * all their attribute, method and parent relationships.
     * </p>
     * 
     * @param job
     *            <code>DotJob</code> that contains the
     *            <code>AnalyzeModel</code> where are the
     *            entities we need to write.
     * @param writer
     *            <code>BufferedWriter</code> to use for
     *            writing.
     * @throws IOExceptionis
     *             thrown if something goes wrong when
     *             accessing the file.
     */
    private void writeEntities(DotJob job,
            BufferedWriter writer) throws IOException {

        /*
         * Get entities from the analyze model of the DotJob
         * and order them alphabetically. If vertical
         * layouting is in use we need to use reversed
         * order.
         */
        Set<String> entityNames = job.analyzeModel
                .getEditor().getEntityNames();
        Vector<String> entityNamesOrdered = new Vector<String>();
        entityNamesOrdered.addAll(entityNames);
        Collections.sort(entityNamesOrdered);
        if (!horizontalLayout) Collections
                .reverse(entityNamesOrdered);

        try {
            for (String entity : entityNamesOrdered) {

                /*
                 * if this entity has type then write it
                 * underneath the entitys name, else just
                 * write the name.
                 */
                String entityLabel = job.analyzeModel
                        .getEditor().getEntityType(entity);
                if (entityLabel != null
                        && entityLabel.length() > 0) {
                    entityLabel = entity + "\\n"
                            + entityLabel;
                } else {
                    entityLabel = entity;
                }

                /*
                 * Replace greater than and lesser than
                 * symbols with '&gt;' and '&lt;'.
                 */
                // entityLabel = entityLabel.replace("<",
                // "&lt;");
                // entityLabel = entityLabel.replace(">",
                // "&gt;");
                /*
                 * write this entity into dot file and check
                 * if it needs to be highlighted.
                 */
                writer.write("entity"
                        + Math.abs(entity.hashCode())
                        + " [label=\"" + entityLabel
                        + "\", ");
                writer
                        .write(colorTheme
                                .getColorString(job.entitiesToBeHighlighted != null
                                        && job.entitiesToBeHighlighted
                                                .contains(entity)));
                writer.write(", fontsize=\"" + fontSize
                        + "\"];");
                writer.newLine();

            }
            for (String entity : entityNamesOrdered) {
                /*
                 * write methods, attributes and parents of
                 * this entity to dot file.
                 */
                writeEntityMethods(job, writer, entity);
                writeEntityAttributes(job, writer, entity);
                writeEntityParents(job, writer, entity);
            }
        } catch (IOException e) {
            // if we caugth IOException just
            // throw it for someone else..
            throw e;
        }
    }

    /**
     * <p>
     * Writes the analyze model to an external file in dot
     * syntax.
     * </p>
     * 
     * @param job
     *            <code>DotJob</code> that contains the
     *            <code>AnalyzeModel</code> we need to
     *            write.
     * @return <code>File</code> containing the analyze
     *         model in dot language. <code>Null</code> is
     *         returned if something went wrong.
     */
    private File createDotFile(DotJob job) {

        File result = new File(
                this.owner
                        .getProperties()
                        .getProperty(
                                GraphicalUI.PROPERTY_TEMP_INPUT_FILE));

        // Initialize stream to file for writing.
        BufferedWriter writer = null;
        try {
            writer = new BufferedWriter(new FileWriter(
                    result));

            try {
                fontSize = Integer
                        .parseInt(this.owner
                                .getProperties()
                                .getProperty(
                                        GraphicalUI.PROPERTY_GRAPH_FONT_SIZE));
            } catch (Exception e) {
            }
            if (fontSize < 5) fontSize = 5;
            else if (fontSize > 100) fontSize = 100;

            // Write starting tag into file and graph-wide
            // attributes.
            writer.write("digraph G { "
                    + colorTheme.getBackgroundColorString()
                    + "; rankdir=\""
                    + (horizontalLayout ? "TB" : "LR")
                    + "\"" + "; fontsize=\"" + fontSize
                    + "\"" + ";");
            writer.newLine();

            // Write the entities and their relationships to
            // the dot file.
            writeEntities(job, writer);

            // Write end tag into file.
            writer.write("}");
            writer.newLine();

            writer.flush();
            writer.close();

        } catch (IOException e) {
            logger.warning(e.getMessage());
            logger.finest(StringTools
                    .exceptionStackTrace(e));
            return null;
        }

        return result;
    }

    /**
     * <p>
     * Method for mapping the current analyze model. This is
     * required to figure out differences with the submodel
     * that needs to be highlighted.
     * </p>
     * 
     * @param job
     *            <code>DotJob</code> where mapping will
     *            be done.
     */
    private void mapCurrentModel(DotJob job) {
        for (String entity : job.entitiesInCurrentModel) {
            for (String method : job.analyzeModel
                    .getEditor().getMethodNames(entity)) {
                for (String target : job.analyzeModel
                        .getEditor()
                        .getEntitiesInfluencedByMethod(
                                entity, method)) {
                    // Method vector stores the method name
                    // plus its owner and
                    // target entities.
                    Vector<String> methodVector = new Vector<String>();
                    methodVector.add(entity);
                    methodVector.add(method);
                    methodVector.add(target);
                    job.methodsInCurrentModel
                            .add(methodVector);
                }
            }

            for (String attribute : job.analyzeModel
                    .getEditor().getAttributeNames(entity)) {
                // Attribute vector stores the owner entity
                // in addition
                // to the attribute entity itself.
                Vector<String> attributeVector = new Vector<String>();
                attributeVector.add(entity);
                attributeVector.add(attribute);
                job.attributesInCurrentModel
                        .add(attributeVector);
            }

            for (String parent : job.analyzeModel
                    .getEditor().getParents(entity)) {
                // Parent vector stores both the parent and
                // inherited entities'
                // names.
                Vector<String> parentVector = new Vector<String>();
                parentVector.add(entity);
                parentVector.add(parent);
                job.parentsInCurrentModel.add(parentVector);
            }
        }
    }

    /**
     * <p>
     * Method for mapping the submodel for highlight
     * request.
     * </p>
     * 
     * @param job
     *            <code>DotJob</code> to be handled.
     * @param drawNewElements
     *            Indicates wether or not the new elements
     *            are supposed to be drawn in the
     *            highlighted model. New elements are those
     *            that exist in the highlight request but do
     *            not exist in the current analyze model.
     */
    private void mapHighlightRequest(DotJob job,
            boolean drawNewElements) {
        for (String entity : job.entitiesRequestedForHighlighting) {
            for (String method : job.highlightModel
                    .getEditor().getMethodNames(entity)) {
                for (String target : job.highlightModel
                        .getEditor()
                        .getEntitiesInfluencedByMethod(
                                entity, method)) {
                    // Method vector stores the method name
                    // plus its owner and
                    // target entities.
                    Vector<String> methodVector = new Vector<String>();
                    methodVector.add(entity);
                    methodVector.add(method);
                    methodVector.add(target);
                    job.methodsRequestedForHighlighting
                            .add(methodVector);
                }
            }

            for (String attribute : job.highlightModel
                    .getEditor().getAttributeNames(entity)) {
                // Attribute vector stores the owner entity
                // in addition
                // to the attribute entity itself.
                Vector<String> attributeVector = new Vector<String>();
                attributeVector.add(entity);
                attributeVector.add(attribute);
                job.attributesRequestedForHighlighting
                        .add(attributeVector);
            }

            for (String parent : job.highlightModel
                    .getEditor().getParents(entity)) {
                // Parent vector stores both the parent and
                // inherited entities'
                // names.
                Vector<String> parentVector = new Vector<String>();
                parentVector.add(entity);
                parentVector.add(parent);
                job.parentsRequestedForHighlighting
                        .add(parentVector);
            }

            if (!job.entitiesInCurrentModel
                    .contains(entity)
                    && !drawNewElements) continue;
            job.entitiesToBeHighlighted.add(entity);
        }
    }

    /**
     * <p>
     * Method for highlighting submodels from the analyze
     * model.
     * </p>
     * 
     * @param highlight
     *            The submodel to be highlighted. If this
     *            argument is <code>null</code>, then all
     *            applied highlights are removed.
     * @param drawNewElements
     *            Indicates wether or not those elements
     *            which do not exist in the current analyze
     *            model should be also drawn and
     *            highlighted.
     */
    public void highlight(AnalyzeModel highlight,
            boolean drawNewElements) {
        DotJob job = new DotJob();
        try {
            job.analyzeModel.getEditor().merge(
                    owner.getControlInterface()
                            .getAnalyzeModel());
        } catch (AnalyzeModelException e) {
            e.printStackTrace();
        }
        job.highlightModel = highlight;

        // Initialize sets for current entities and their
        // relationships
        // and then map those entities and their
        // relationships which are
        // requested to be highlighted.
        job.entitiesInCurrentModel = owner
                .getControlInterface().getAnalyzeModel()
                .getEditor().getEntityNames();
        if (highlight != null) job.entitiesRequestedForHighlighting = highlight
                .getEditor().getEntityNames();

        // If highlight is null, then just remove any
        // highlightings that
        // are currently applied.
        if (highlight != null) {

            mapCurrentModel(job);

            mapHighlightRequest(job, drawNewElements);

            // Calculate methods to be highlighted.
            for (Vector<String> method : job.methodsRequestedForHighlighting) {
                if (!job.methodsInCurrentModel
                        .contains(method)
                        && !drawNewElements) continue;
                job.methodsToBeHighlighted.add(method);
            }

            // Calculate attributes to be highlighted.
            for (Vector<String> attribute : job.attributesRequestedForHighlighting) {
                if (!job.attributesInCurrentModel
                        .contains(attribute)
                        && !drawNewElements) continue;
                job.attributesToBeHighlighted
                        .add(attribute);
            }

            // Calculate parents to be highlighted.
            for (Vector<String> parent : job.parentsRequestedForHighlighting) {
                if (!job.parentsInCurrentModel
                        .contains(parent)
                        && !drawNewElements) continue;
                job.parentsToBeHighlighted.add(parent);
            }
        }

        updateModel(job);

        // Destroy class sets associated with highlighting.
    }

    /**
     * <p>
     * Method for highlighting submodels from the analyze
     * model. As default we assume that new entities don't
     * need to be drawn.
     * </p>
     * 
     * @param highlight
     *            Submodel to be highlighted.
     */
    public void highlight(AnalyzeModel highlight) {
        highlight(highlight, false);
    }

    /**
     * <p>
     * Method for updating the model. This method pushes the
     * given dot job to the top of the dot job stack and
     * notifies the running thread about it. If no updation
     * thread is running yet, one is created.
     * </p>
     * 
     * @param job
     *            <code>DotJob</code> to be run next.
     */
    private void updateModel(final DotJob job) {
        synchronized (DOT_JOB_STACK) {
            DOT_JOB_STACK.push(job);
        }

        if (EXECUTING_DOT_THREAD == null) {
            EXECUTING_DOT_THREAD = new Thread(
                    new Runnable() {
                        public void run() {
                            executeDot();
                        }
                    });
            EXECUTING_DOT_THREAD.start();
        } else {
            synchronized (EXECUTING_DOT_THREAD) {
                EXECUTING_DOT_THREAD.notify();
            }
        }
    }

    /**
     * <p>
     * Method for updating the analyze model.
     * </p>
     * 
     * @param analyzeModel
     *            New <code>AnalyzeModel</code>.
     */
    public void updateModel(AnalyzeModel analyzeModel) {
        DotJob job = new DotJob();
        try {
            job.analyzeModel.getEditor()
                    .merge(analyzeModel);
        } catch (AnalyzeModelException e) {
            e.printStackTrace();
        }
        updateModel(job);
    }

    /**
     * <p>
     * This finalize method makes sure the thread running
     * for panel update shuts down.
     * </p>
     */
    protected void finalize() throws Throwable {
        THREAD_RUNNING = false;
        synchronized (EXECUTING_DOT_THREAD) {
            if (EXECUTING_DOT_THREAD != null) {
                EXECUTING_DOT_THREAD.notify();
            }
        }
        synchronized (EXECUTING_EXPORT_THREAD) {
            if (EXECUTING_EXPORT_THREAD != null) {
                EXECUTING_EXPORT_THREAD.notify();
            }
        }
        super.finalize();
    }

    /**
     * <p>
     * This method draws the graph generated by dot to the
     * panel's canvas.
     * </p>
     */
    public void paintComponent(Graphics g) {
        super.setBackground(colorTheme
                .getBackgroundColorAsJavaObject());
        super.paintComponent(g);
        int x = 0, y = 0;
        // Update component size to match the scroller size.
        if (this.modelImage != null) {
            x = Math.max(0, (int) Math.floor((this
                    .getWidth() - this.modelImage
                    .getWidth()) / 2));
            y = Math.max(0, (int) Math.floor((this
                    .getHeight() - this.modelImage
                    .getHeight()) / 2));

            g.drawImage(this.modelImage, x, y, this);

            // set preferred size same size as the image and
            // make sure
            // that panel is drawn again.
            this.setPreferredSize(new Dimension(
                    this.modelImage.getWidth(),
                    this.modelImage.getHeight()));
            this.revalidate();
        }
    }

    /**
     * <p>
     * This method should be executed in its own thread.
     * This thread keeps running until
     * <code>threadRunning</code> class variable is set to
     * false.
     * </p>
     * <p>
     * It takes the latest job from the dot job stack and
     * disposes all the other jobs in the stack at the same
     * time. The newest job is then modeled and drawn to the
     * canvas. After that the thread sleeps until waken
     * again by notification [<code>executing.notify()</code>].
     * <p>
     */
    private void executeDot() {

        while (THREAD_RUNNING) {
            if (!DOT_JOB_STACK.isEmpty()) {

                long startTime = System.currentTimeMillis();
                DotJob job = null;

                synchronized (DOT_JOB_STACK) {
                    job = DOT_JOB_STACK.pop();
                    DOT_JOB_STACK.clear();
                }

                if (job == null) continue;

                File dotFile = createDotFile(job);
                if (dotFile == null) {
                    logger
                            .warning("Error in initializing dotFile, exiting updateAnalyzemodel.");
                    continue;
                }

                File imageFile = runDot(dotFile);
                if (imageFile == null) {
                    logger
                            .warning("Error in initializing imageFile, exiting updateAnalyzemodel.");
                    continue;
                }

                // Check if image file exists... just to be
                // sure.
                if (!imageFile.exists()) {
                    logger
                            .warning("DotPanel.java: imageFile ("
                                    + imageFile.getName()
                                    + ") not found.");
                    continue;
                }

                // If everything went ok, load image from
                // file.
                try {
                    this.modelImage = ImageIO
                            .read(imageFile);
                } catch (IOException e) {
                    logger
                            .warning("DotPanel.java: Error in reading image from file.");
                    logger.warning(StringTools
                            .exceptionStackTrace(e));
                    continue;
                }

                // Remove temp files
                // dotFile.delete();
                // imageFile.delete();

                this.repaint();

                // Uncomment the next line of code to
                // confirm that the dot panel
                // really
                // is running in a separate thread.
                // for ( int i = 0; i < 100000;
                // System.out.println(++i) );

                timeElapsed = System.currentTimeMillis()
                        - startTime;
                if (timerComponent != null) {
                    timerComponent.setText(" "
                            + timeElapsed + " ms ");
                }
            } else {
                try {
                    synchronized (EXECUTING_DOT_THREAD) {
                        EXECUTING_DOT_THREAD.wait();
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
    }

    /**
     * <code>JLabel</code> where the updation time of this
     * <code>DotPanel</code> is drawn.
     * 
     * @param timerComponent
     *            <code>JLabel</code> on which you need to
     *            get the updation time to.
     */
    public void setTimerComponent(JLabel timerComponent) {
        this.timerComponent = timerComponent;
    }
}
