/*
 *  Copyright (C) 2006  UCOT
 *
 *  Distributed under the terms of the MIT License.
 *  See "LICENCE" or http://www.opensource.org/licenses/mit-license.php for details.
 */
package ucot.ui.gui.tree.analyzemodeltree;

import java.util.Collections;
import java.util.List;
import java.util.Set;
import java.util.Vector;

import javax.swing.Icon;

/**
 * <p>
 * This class represents all <tt>TreeItems<tt> in the
 * <tt>AnalyzeModelTrre</tt>. It gathers common functionality
 * shared among <tt>TreeItems</tt>s that represents entities
 * in different levels in the tree.</p>
 * 
 * @author UCOT
 */
public abstract class AbstractEntityTreeItem extends
        TreeItem {

    /**
     * Should this tree node be expandaple (should it show
     * its childs).
     */
    protected boolean expand;

    /**
     * Constructor for the class.
     * 
     * @param name
     *            The name of the entity.
     * @param model
     *            The model where the entity resides.
     * @param parent
     *            The parent node of this
     *            <code>TreeItem</code>.
     */
    public AbstractEntityTreeItem(String name,
            AnalyzeTreeModel model, TreeItem parent) {
        super(name, model, parent);
        this.expand = true;
    }

    /*
     * (non-Javadoc)
     * 
     * @see ucot.ui.gui.tree.analyzemodeltree.TreeItem#getChildren()
     */
    public List<TreeItem> getChildren() {
        if (expand
                && getAnalyzeModel().getEditor()
                        .containsEntity(getName())) {
            List<TreeItem> items = new Vector<TreeItem>();
            items.addAll(this.getChildTreeItems());
            items.addAll(this.getAttributeTreeItems());
            items.addAll(this.getMethodTreeItems());

            return items;
        } else return new Vector<TreeItem>();
    }

    /**
     * Returns the <code>TreeItem</code>s for the
     * entity's methods.
     * 
     * @return List of <code>MethodTreeItem</code>s
     *         representing entity's method.
     */
    @SuppressWarnings("unchecked")
    // Suppress unchecked type warnings.
    protected List<MethodTreeItem> getMethodTreeItems() {
        List<MethodTreeItem> items = new Vector<MethodTreeItem>();
        Set<String> methods = getAnalyzeModel().getEditor()
                .getMethodNames(getName());
        for (String method : methods) {
            MethodTreeItem methodItem = new MethodTreeItem(
                    getName(), method, getTreeModel(), this);
            items.add(methodItem);
        }

        Collections.sort(items);
        Collections.reverse(items);

        return items;
    }

    /**
     * Returns the <code>TreeItem</code>s for the
     * entity's attributes.
     * 
     * @return List of <code>AttributeTreeItem</code>s
     *         representing entity's attributes.
     */
    @SuppressWarnings("unchecked")
    // Suppress unchecked type warnings.
    protected List<AttributeTreeItem> getAttributeTreeItems() {
        List<AttributeTreeItem> items = new Vector<AttributeTreeItem>();
        Set<String> attributes = getAnalyzeModel()
                .getEditor().getAttributeNames(getName());
        for (String attribute : attributes) {
            items.add(new AttributeTreeItem(attribute,
                    getTreeModel(), this));
        }

        Collections.sort(items);
        Collections.reverse(items);
        return items;
    }

    /**
     * Returns the <code>TreeItem</code>s for the
     * entity's childs.
     * 
     * @return List of <code>ChildTreeItem</code>s
     *         representing entity's childs.
     */
    @SuppressWarnings("unchecked")
    // Suppress unchecked type warnings.
    protected List<ChildTreeItem> getChildTreeItems() {
        List<ChildTreeItem> items = new Vector<ChildTreeItem>();
        Set<String> children = getAnalyzeModel()
                .getEditor().getChildren(getName());
        for (String child : children) {
            items.add(new ChildTreeItem(child, this
                    .getTreeModel(), this));
        }

        Collections.sort(items);
        Collections.reverse(items);

        return items;
    }

    @Override
    public abstract Icon getIcon();
}
