/*
 *  Copyright (C) 2006  UCOT
 *
 *  Distributed under the terms of the MIT License.
 *  See "LICENCE" or http://www.opensource.org/licenses/mit-license.php for details.
 */
package ucot.ui.gui.tree.analyzemodeltree;

import java.util.Collections;
import java.util.List;
import java.util.Set;
import java.util.Vector;

import javax.swing.Icon;

import ucot.model.ModelEditor;

/**
 * <p>
 * This represents the entities <code>TreeItem</code> that
 * contains all the entities of the analyze model.
 * </p>
 * 
 * @author pajumasu
 * 
 */
public class EntitiesTreeItem extends TreeItem {

    /**
     * <p>
     * Constructs the object.
     * </p>
     * 
     * @param model
     *            The <code>TreeModel</code> this item
     *            belongs to.
     * @param parent
     *            The parent node.
     */
    public EntitiesTreeItem(AnalyzeTreeModel model,
            TreeItem parent) {
        super("Entities", model, parent);
    }

    @Override
    public Icon getIcon() {
        return AnalyzeModelTree.entitiesIcon;
    }

    /*
     * (non-Javadoc)
     * 
     * @see ucot.ui.gui.tree.analyzemodeltree.TreeItem#getChildren()
     */
    @SuppressWarnings(value = { "unchecked" } // Suppreess
                                                // type
                                                // safty
    // checking.
    )
    @Override
    public List getChildren() {

        ModelEditor editor = getAnalyzeModel().getEditor();
        List<EntityTreeItem> children = new Vector<EntityTreeItem>();
        Set<String> entities = editor.getEntityNames();

        List<String> childEntities = new Vector<String>();
        List<String> leafEntities = new Vector<String>();

        // Go trough list of entities and check if they are
        // leaf entities or not. Leaf entities are placed
        // into
        // own list and not-Leaf entities are put into list
        // of
        // childEntities.
        for (String entity : entities) {

            if (editor.getMethodNames(entity).size() == 0
                    && editor.getAttributeNames(entity)
                            .size() == 0
                    && editor.getChildren(entity).size() == 0) {

                leafEntities.add(entity);
            } else {
                childEntities.add(entity);
                children.add(new EntityTreeItem(entity,
                        getTreeModel(), this));
            }
        }

        // Check if leaf entities are referenced by any of
        // the
        // child entities, if not put those also into list
        // of
        // clid entities.
        for (String leafEntity : leafEntities) {

            boolean isLeafEntityReferenced = false;
            for (String childEntity : childEntities) {
                if (isEntityInfluencingEntity(childEntity,
                        leafEntity)) {
                    isLeafEntityReferenced = true;
                }
            }
            if (!isLeafEntityReferenced) {
                children.add(new EntityTreeItem(leafEntity,
                        getTreeModel(), this));
            }
        }

        // Sort entities in alphabetical order
        Collections.sort(children);
        Collections.reverse(children);

        return children;

    }

    /**
     * Retuns true is targetEntity is referenced by
     * sourceEntity. TargetEntity os referenced if it is
     * sourceEntitys attribute, child or is target of
     * sourceEntitys methods.
     * 
     * @param sourceEntity
     *            name of the entity that is influencing.
     * @param targetEntity
     *            name of the entity that is influenced.
     * @return true if sourceEntity is influencing
     *         targetEntity.
     */
    private boolean isEntityInfluencingEntity(
            String sourceEntity, String targetEntity) {

        ModelEditor editor = this.getAnalyzeModel()
                .getEditor();

        // Check if targetEntity is target of any of the
        // sourceEntitys methods.
        Set<String> methods = editor
                .getMethodNames(sourceEntity);
        for (String method : methods) {
            Set<String> influencedEntities = editor
                    .getEntitiesInfluencedByMethod(
                            sourceEntity, method);
            for (String influencedEntity : influencedEntities) {
                if (targetEntity.equals(influencedEntity)) { return true; }
            }
        }

        // Check if targetEntity is sourceEntitys attribute.
        Set<String> attributes = editor
                .getAttributeNames(sourceEntity);
        for (String attribute : attributes) {
            if (targetEntity.equals(attribute)) { return true; }
        }

        // Check if targetEntity is sourceEntitys child.
        Set<String> children = editor
                .getChildren(sourceEntity);
        for (String child : children) {
            if (targetEntity.equals(child)) { return true; }
        }

        // return false if no influence
        // between entities was found.
        return false;
    }
}
