/*
 *  Copyright (C) 2006  UCOT
 *
 *  Distributed under the terms of the MIT License.
 *  See "LICENCE" or http://www.opensource.org/licenses/mit-license.php for details.
 */
package ucot.ui.gui.tree.analyzemodeltree;

import java.util.List;
import java.util.Vector;

import javax.swing.Icon;

import ucot.model.AnalyzeModel;

/**
 * <p>
 * <code>TreeItem</code> is basic building block of
 * <code>AnalyzeTreeModel</code>. Its subclasses are used
 * to create dynmic internal model of the analyzemodel which
 * can be used like a tree. By using these classes
 * <code>AnalyzeTreeModel</code> can serve the actual tree
 * representing the analyze model.
 * </p>
 * 
 * @see javax.swing.JTree
 * @see ucot.model.AnalyzeModel
 */
public abstract class TreeItem implements Comparable {
    /**
     * The name of the item. This should be shown by the
     * tree
     */
    private String name;

    /**
     * The model this <code>TreeItem</code> fetches its
     * data (mostly childs).
     */
    private AnalyzeTreeModel model;

    /** The parent <code>TreeItem</code> of this node. */
    private TreeItem parent;

    /**
     * <p>
     * Constructs the object.
     * </p>
     * 
     * @param name
     *            The name of the item.
     * @param model
     *            The <code>TreeModel</code> this item
     *            belongs to.
     * @param parent
     *            The parent node for this item.
     */
    public TreeItem(String name, AnalyzeTreeModel model,
            TreeItem parent) {
        this.name = name;
        this.model = model;
        this.parent = parent;
    }

    public AnalyzeTreeModel getTreeModel() {
        return model;
    }

    /**
     * Returns the analyze model used by this item.
     * 
     * @return the analyze model used by this item.
     * @see AnalyzeModel
     */
    public AnalyzeModel getAnalyzeModel() {
        if (model != null) return model.getAnalyzeModel();
        else return null;
    }

    /**
     * Returns the name of this item.
     * 
     * @return the name of this item.
     */
    public String getName() {
        return name;
    }

    /**
     * Returns the name of this item.
     * 
     * @see #getName()
     */
    public String toString() {
        return getName();
    }

    /**
     * Returns the child items of this item.
     * 
     * @return The childs.
     */
    public List<TreeItem> getChildren() {
        return new Vector<TreeItem>();
    }

    /**
     * <p>
     * Returns the path to this item inside the tree. Path
     * is list of tree items where the most higest parent is
     * first and the node ask to return the path is last in
     * the list.
     * </p>
     * 
     * @return List of <code>TreeItem</code> which
     *         represents the path to this item.
     */
    public List<TreeItem> getPath() {
        List<TreeItem> path = new Vector<TreeItem>();
        path.add(0, this);
        if (parent != null) {
            path.addAll(0, parent.getPath());
        }
        return path;
    }

    /**
     * Returns the parent item of this item.
     * 
     * @return the parent item of this item.
     */
    public TreeItem getParent() {
        return this.parent;
    }

    /**
     * Returns <code>Icon</code> for this
     * <code>TreeItem</code>.
     * 
     * @return <code>Icon</code> for this
     *         <code>TreeItem</code>.
     */
    public abstract Icon getIcon();

    /**
     * Compares this treeitem to another
     * 
     * @param other
     *            treeitem to compare to
     * @return 0 if the treeitems are equal
     */
    public int compareTo(Object other) {
        if (other instanceof TreeItem) { return ((TreeItem) other)
                .getName().compareTo(this.getName()); }
        return -1;
    }
}
