/*
 *  Copyright (C) 2006  UCOT
 *
 *  Distributed under the terms of the MIT License.
 *  See "LICENCE" or http://www.opensource.org/licenses/mit-license.php for details.
 */
package ucot.utils;

import java.io.File;
import java.util.Vector;

import javax.swing.filechooser.FileFilter;

/**
 * <p>
 * CustomFileFilter for JFileChooser that accepts extensions
 * that are given in constructor.
 * <p>
 * <p>
 * How to use:<br/>
 * 
 * <pre><code>
 * JFileChooser f = new JFileChooser(&quot;.&quot;);
 * f.addChoosableFileFilter(new CustomFileFilter(
 *         &quot;.jpg , .jpeg&quot;, new String[] { &quot;jpg&quot;, &quot;jpeg&quot; }));
 * </code></pre>
 * 
 * </p>
 * 
 * @author ilanliuk
 * 
 */
public class CustomFileFilter extends FileFilter {

    private String description;

    private Vector<String> extensions = new Vector<String>();

    /**
     * Creates new CustomFileFilter for filtering files in
     * JFileChooser.
     * 
     * @param description
     *            short description of what filter accepts
     *            (for example "image files" etc.)
     * @param acceptableExtensions
     *            array of extensions that are accepted by
     *            filter
     */
    public CustomFileFilter(String description,
            String[] acceptableExtensions) {
        super();
        this.description = description;
        this.parseExtensions(acceptableExtensions);
    }

    /**
     * Creates vector from given String-array
     * 
     * @param extensions
     *            aray of String
     */
    private void parseExtensions(String[] extensions) {

        // add contents of array to vector
        for (int i = 0; i < extensions.length; i++) {
            this.extensions.add(extensions[i]);
        }
    }

    @Override
    public boolean accept(File f) {

        // cause we don't want to hide directories
        // return true if f is directory.
        if (f.isDirectory()) { return true; }

        // strip extension from file
        String file = f.getName();
        String extension = (file.substring(Math.max(file
                .lastIndexOf("."), 0))).toLowerCase();

        // go through extensions vector and check
        // if files extension is acceptable
        for (String ext : this.extensions) {
            if (extension.equals("." + ext)) { return true; }
        }

        // if no match was found
        // return false
        return false;
    }

    @Override
    public String getDescription() {
        String filters = " (";
        for (String extension : extensions) {
            filters = filters + "*." + extension + ",";
        }
        filters = filters
                .substring(0, filters.length() - 1)
                + ")";
        return this.description + filters;
    }

    /**
     * Returns extensions this FileFilter accepts in a
     * vector.
     * 
     * @return Vector of acceptable extensions
     */
    public Vector<String> getExtensions() {
        Vector<String> result = new Vector<String>();
        result.addAll(extensions);
        return result;
    }
}
