/*
 *  Copyright (C) 2006  UCOT
 *
 *  Distributed under the terms of the MIT License.
 *  See "LICENCE" or http://www.opensource.org/licenses/mit-license.php for details.
 */
package ucot.utils;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URL;

import ucot.Messages;

/**
 * <p>
 * This class contains some operations for file handling.
 * </p>
 * 
 */
public class FileTools {
    /** Message for file not found -situation. */
    public static final String FILE_NOT_FOUND = Messages
            .getString("FILE_NOT_FOUND");

    /** Message for file not readable -situation. */
    public static final String FILE_NOT_READABLE = Messages
            .getString("FILE_NOT_READABLE");

    /** Message for file not writable -situation. */
    public static final String FILE_NOT_WRITABLE = Messages
            .getString("FILE_NOT_WRITABLE");

    /** Message for operation not permited -situation. */
    public static final String OPERATION_FORBIDDEN = Messages
            .getString("OPERATION_FORBIDDEN");

    /**
     * Method copies file. It copies from source to target
     * file. If the target file exists its is overwriten. If
     * the target file is direcoty then the source file is
     * copied with its original name under the target
     * directory.
     * 
     * @param from
     *            The source file to copy from. It must
     *            exist.
     * @param to
     *            The target file or directory.
     * @throws IOException
     */
    public static void copyFile(File from, File to)
            throws IOException {
        if (!from.exists()) { throw new IOException(
                Messages
                        .formatMessage(FILE_NOT_FOUND, from)); }
        if (!from.isFile()) { throw new IOException(
                OPERATION_FORBIDDEN); }
        if (!from.canRead()) { throw new IOException(
                Messages.formatMessage(FILE_NOT_READABLE,
                        from)); }

        if (to.isDirectory()) to = new File(to, from
                .getName());

        if (to.exists()) {
            if (!to.canWrite()) { throw new IOException(
                    Messages.formatMessage(
                            FILE_NOT_WRITABLE, to)); }
        } else {
            File dir = new File(to.getParent());

            if (!dir.isDirectory() || !dir.exists()
                    || !dir.canWrite()) { throw new IOException(
                    Messages.formatMessage(
                            FILE_NOT_WRITABLE, to)); }

            to.createNewFile();
        }

        FileInputStream input = null;
        FileOutputStream output = null;

        try {
            try {
                input = new FileInputStream(from);
                output = new FileOutputStream(to);

                final int bufferSize = 5120;
                byte[] buffer = new byte[bufferSize];
                int read = 0;

                while ((read = input.read(buffer, 0,
                        bufferSize)) != -1) {
                    output.write(buffer, 0, read);
                }
            } finally {
                if (input != null) input.close();
                if (output != null) output.close();
            }
        } catch (IOException e) {
            throw e;
        }
    }

    /**
     * Copies file represented by url to the the file
     * represented by anaother url. Uses
     * <code>copyFile(File,File)</code>-method to perform
     * the actual copy.
     * 
     * @param from
     *            The URL of source file.
     * @param to
     *            The URL of target file or directory.
     * @throws IOException
     * @see #copyFile(File, File)
     */
    public static void copyFile(URL from, URL to)
            throws IOException {
        copyFile(from.getFile(), to.getFile());
    }

    /**
     * Copies file represented by path to target path. Uses
     * <code>copyFile(File,File)</code>-method to perform
     * the actual copy.
     * 
     * @param from
     *            The path of the source file.
     * @param to
     *            The path of the target file or directory.
     * @throws IOException
     * @see #copyFile(File, File)
     */
    public static void copyFile(String from, String to)
            throws IOException {
        copyFile(new File(from), new File(to));
    }

    /**
     * Changes given file's extension
     * 
     * @param original
     *            original file
     * @param extension
     *            new extension for the file
     * @return file with changed extension
     */
    public static File changeExtension(File original,
            String extension) {
        // If its not a file, we dont want it
        // if (!original.isFile()) return original;

        String temp = original.toString();

        // Find where the extension begings
        int pos = temp.lastIndexOf(".");
        if (pos < 0) return original;

        return new File(temp.substring(0, pos + 1)
                + extension);
    }
}
