/*
 *  Copyright (C) 2006  UCOT
 *
 *  Distributed under the terms of the GPLv2 License.
 *  See "LICENCE" or http://www.opensource.org/licenses/gpl-license.php for details.
 */
package ucot.parser;

import java.io.File;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.Vector;
import java.util.logging.Logger;

import ucot.ModuleProperties;
import ucot.exceptions.BadPropertyValueException;
import ucot.input.UseCase;
import ucot.input.UseCaseStep;
import edu.stanford.nlp.parser.lexparser.LexicalizedParser;
import edu.stanford.nlp.trees.GrammaticalRelation;
import edu.stanford.nlp.trees.GrammaticalStructure;
import edu.stanford.nlp.trees.GrammaticalStructureFactory;
import edu.stanford.nlp.trees.Tree;
import edu.stanford.nlp.trees.TreeGraphNode;
import edu.stanford.nlp.trees.TreebankLanguagePack;
import edu.stanford.nlp.trees.TypedDependency;
import edu.stanford.nlp.util.Filters;
import ucot.parser.*;

/**
 * Parser that uses Stanford parser to parse te sentence.
 * @author panu
 *
 */
public class StanfordAdapter extends ModuleProperties implements ParserInterface {
	
	public static final String name = "The Stanford parser";
	
	private Logger logger = Logger.getLogger("ucot.parser");
	
	public final static String PARSER_FILE = "parser_file";	

	private boolean optionsChanged = true;
	LexicalizedParser parser;
	private GrammaticalStructureFactory gsf;
	
	public StanfordAdapter(String pluginRootDir){
		logger.info("StandfrodAdapter created.");
		// Set default parser file for Stanford.
		Properties options = new Properties();
		File parserFile = new File(pluginRootDir, "englishPCFG.ser.gz");
		options.setProperty(PARSER_FILE, parserFile.toString());
		setProperties(options);
		updateFromOptions();
	}
	
	public StanfordAdapter() {
		this(".");
	}
	
	public String getName() {
		return name;
	}
	
    public String toString() {
        return this.getName();
    }
	
	/**
	 * Updates internal parser field to match given options.
	 */
	private void updateFromOptions(){
		// Creare new parser if options have been changed 
		// or we dont have current paresr.
		if (optionsChanged || null == parser){
			logger.fine("Creating Standford parser.");
			String parserFile = getProperties().getProperty("parser_file");
			if (null == parserFile ){
				this.parser = new LexicalizedParser();
			} else {
				logger.fine("Using parser file " + parserFile);
				this.parser = new LexicalizedParser(parserFile);
			}
			TreebankLanguagePack tlp = parser.getOp().tlpParams.treebankLanguagePack();
			gsf = tlp.grammaticalStructureFactory(Filters.acceptFilter());
			
			optionsChanged = false;
			logger.fine("Standford parser created.");
		}
	}
	
	
	private void printTree(Tree startTree, List<Tree> parents) {
		Vector<Tree> copy = new Vector<Tree>();
		copy.addAll(parents);
		for(Tree tree : startTree.children()){
			if (tree.isLeaf()){
				System.out.println(tree);
				for(Tree t : parents){
					System.out.println("           " + t.label());
				}
			}
			copy.add(tree);
			printTree(tree, copy);
		}
	}
	
	private static final Set<String> ignoreInNP = new HashSet<String>();
	private static final String[]ignoreArray = new String[]{"DT"};
	static{
		for(String str : ignoreArray){
			ignoreInNP.add(str);
		}
	}
	/**
	 * Builds noun phrase that the leaf represents.
	 * If leaf is NP-branch then the whole noun phrase
	 * the branc represents is returned. 
	 * If leaf is just a leaf then the value of theleaf is returned.
	 * @param leaf 
	 * @param root The root of the whole tree.
	 * @return The noun phrase that the leaf belongs to.
	 */
	private String buildNounPhrase(Tree leaf, Tree root){
		if (leaf.isLeaf()){
			if (leaf.label().toString().equals("-LRB-")){
				return "(";
			} else if (leaf.label().toString().equals("-RRB-")){
				return ")";
			} else return leaf.label().toString();
		}
		else {
			// This is NP-branch so recursively build the whole noun phrase.
			Tree NP = leaf;
			List<Tree> childs = NP.getChildrenAsList();
			StringBuffer completeNoun = new StringBuffer();
			for(Tree child : childs){
				if (! ignoreInNP.contains(child.label().toString())){
					completeNoun.append(buildNounPhrase(child,root));
					completeNoun.append(" ");
				}
			}
			String noun = completeNoun.toString().trim().toLowerCase();
			return noun;
		}
	}
	/**
	 * Returns the highest branch called by spesific name.
	 * @param name The name we are interested of.
	 * @param leaf The leaf we start looking from.
	 * @param root The root of the whole tree.
	 * @return Null if no such branch is found, otherwise highest brnach called name is returned.
	 */
	private Tree getHighest(String name, Tree leaf, Tree root){
		Tree ret = null;
		Tree runner = leaf;
		while(runner != null){
			if (runner.label().toString().endsWith(name)){
				ret = runner;
			}
			runner = runner.parent(root);
		}
		return ret;		
	}
	
	/**
	 * Returns first branch which name exists in given array.
	 * The tree is traveled from the given leaf towards the root.
	 * @param names The array of names we are interested in.
	 * @param leaf The leaf searching starts.
	 * @param root The root of the whole tree.
	 * @return Null if not found.
	 */
	private Tree getFirst(String[] names, Tree leaf, Tree root) {
		Tree runner = leaf;
		List<String> namesCollection = new Vector<String>();
		for(String name : names){
			namesCollection.add(name);
		}
		return getFirst(namesCollection, leaf, root);
	}
	
	/**
	 *  Returns first branch which name exists in given collection.
	 * The tree is traveled from the given leaf towards the root.
	 * @param names The collection of names we are interested in.
	 * @param leaf The leaf searching starts.
	 * @param root The root of the whole tree.
	 * @return Null if not found.
	 */
	private Tree getFirst(Collection<String> names, Tree leaf, Tree root){
		Tree runner = leaf;
		while(runner != null){
			if (names.contains(runner.label().toString())){
				return runner;
			}
			runner = runner.parent(root);
		}
		return null;
	}
	
	/* (non-Javadoc)
	 * @see ucot.parser.ParserInterface#parse(ucot.input.UseCase)
	 */
	public synchronized ParsedData parse(UseCase useCase) {
		updateFromOptions();
		ParsedData parsedData = new ParsedData();
		parsedData.setUseCase(useCase);
		useCase.setParser(this);
		logger.info("Parsing usecase '" + useCase.getName()+"'.");
		
		// Parse step by step.
		for(UseCaseStep step : useCase){
			
			
			
			boolean parsed = parser.parse(step.getStep());
			if (parsed){
				logger.finer("Parsed succesfully sentence: " + step);
			} else {
				logger.warning("Parsed failed for sentence: " + step);
			}
			


			// Ask Stanford to parse the sentence.
			Tree tree = parser.getBestParse();
			
			// Create empty sentence to represent parsed sentence.
			Sentence sentence = new Sentence();			
			
			Collection<Tree> leaves = tree.getLeaves();
			logger.fine("Leaves are " + leaves);
			Map<String,Word> nameToWordMap= new HashMap<String,Word>();
			
			Collection<String> types = new HashSet<String>();
			types.add("NP");
			types.add("VP");
			
			Map<String,Word> nounToWordMap= new HashMap<String,Word>();
			int index = 1;
			
			// Go through the trees leafes.
			for(Tree leaf : leaves){
				String token = leaf.label().toString();
				if (token.length()>0){
					String name = token+"-"+index++;
					
					Word word = null;

			
					
					// Check whether we are hadling noun phrase or verb phrase.
					Tree type = getFirst(types, leaf, tree);
					
					if (type != null){						
						String typeName = type.label().toString();

						if (typeName.equals("NP")){
							// NP is a multiword noun (like insurance company).
							// So we need to buld the whole noun.
							Tree npLeaf = getHighest("NP", leaf, tree);
							String noun = buildNounPhrase(npLeaf,tree);
							
							// Check if we have already build word for this noun.
							// It is important that we have ontly one word to 
							// represent the same noun.
							word = nounToWordMap.get(noun);
							if (null == word){
								// Not yet. So build it.
								word = new Word(noun,"");
								word.setWordClass(ParserInterface.NOUN);
								nounToWordMap.put(noun, word);
								sentence.addWord(word);
							}
						}
						else if (typeName.equals("VP")){
							// This is verb phrase so mark the word
							// as verb too.
							word = new Word(token.toLowerCase(),"");
							word.setWordClass(ParserInterface.VERB);
							sentence.addWord(word);
						}
					} 
					if (word == null){
						// If we have a word then add it to the sentence
						word = new Word(token.toLowerCase(),"");
						sentence.addWord(word);						
					}
					// Add name to the word mapping.
					nameToWordMap.put(name, word);

				}
				
				
			}
			logger.fine("Line is " + step.getStep());
			logger.fine("Sentence is " + sentence);
			logger.fine("Name to Word map is " + nameToWordMap);
			
			
			GrammaticalStructure gs = gsf.newGrammaticalStructure(tree);
			// Loop trough all depencies.
			Collection<TypedDependency> typedDepencies = gs.allTypedDependencies();			
			for(TypedDependency typedDependency : typedDepencies){
				GrammaticalRelation grammaticalRel = typedDependency.reln();
				
				
				TreeGraphNode gov = typedDependency.gov();
				TreeGraphNode dep =	typedDependency.dep();
				String govStr = gov.label().toString();
				String depStr = dep.label().toString();
				
				Word govWord = nameToWordMap.get(govStr); 
				Word depWord = nameToWordMap.get(depStr);
				
				GrammaticalRelation gr = grammaticalRel;
				Set<String> relations = new HashSet<String>();
				while(gr != null){
					relations.add(gr.toString());
					gr = gr.parent();
				}
				logger.fine("("+govStr+") " + govWord + "- "+relations+" -> " + "("+depStr+") "+depWord);
				
				 if(relations.contains("subj")){
					
					// There is linkage between verb and subject
					govWord.addLink(new Link(ParserInterface.SUBJECT,depWord));
				} else if(relations.contains("obj")){
					// Loop throug the depencies so that we find the orginal
					// verb.
					while(gov != null && ! govWord.getWordClass().equals(ParserInterface.VERB)){
						govWord = nameToWordMap.get(gov.label().toString()); 
						depWord = nameToWordMap.get(dep.label().toString());
						dep = gov;
						gov = gs.getGovernor(dep);						
					}
					
					// There is linkage between verb and subject
					govWord.addLink(new Link(ParserInterface.OBJECT,depWord));
				} else if(relations.contains("conj") && depWord.getWordClass().equals(ParserInterface.VERB)){
					// This means that dep is in conjuncted verb so we need
					// to find the subject of its gov so we can set
					// the right subject for this word.					
					TreeGraphNode conj = gs.getGovernor(gov);
					Link subject = govWord.getLink(ParserInterface.SUBJECT);					
					if (subject != null){
						depWord.addLink(new Link(ParserInterface.SUBJECT,subject.getTo()));
					}
				} else if (relations.contains("prep") && govWord.getWordClass().equals(ParserInterface.VERB)){
					Set<TreeGraphNode> deps = gs.getDependents(dep); 
					for(TreeGraphNode node : deps){
						Word word = nameToWordMap.get(node.label().toString());
						if (null != word){
							govWord.addLink(new Link("",word));
						}
					}


				}						
			}
			logger.fine("Final sentence is " + sentence);
			parsedData.addSentence(sentence);
		}	
		logger.info("Parsed usecase '" + useCase.getName()+"'.");
		return parsedData;
		
	}
	
	/* (non-Javadoc)
	 * @see ucot.ModulePropertyInterface#loadDefaultProperties()
	 */
	public Properties loadDefaultProperties() {
		Properties result = super.loadDefaultProperties();		
		result.setProperty(PARSER_FILE, "englishPCFG.ser.gz");
		// TODO Set default properties here, for example:
		// result.setProperty("SETTING_1", "Woot");
		// result.setProperty("SETTING_2", "WTF?");
		return result;
	}

	/* (non-Javadoc)
	 * @see ucot.ModulePropertyInterface#applyProperties()
	 */
	public void applyProperties() throws BadPropertyValueException {
		// TODO Auto-generated method stub
		
	}
	
}
